/******************************************************************************/
/*                                                                            */
/*  Version: 1.00                                                             */
/*  Date: September 1, 2005                                                   */
/*  Code Written By: Rudy Frahm                                               */
/*                   Southwest Research Institute                             */
/*                   PO Drawer 28510                                          */
/*                   San Antonio, TX  78228                                   */
/*                   Office: (210) 522-3855                                   */
/*                   FAX: (210) 647-4325                                      */
/*                   E-mail: rfrahm@swri.edu                                  */
/*                                                                            */
/*  Purpose: This program reads the AXIS1 binary files and prints them.       */
/*           AXIS1 data contains measurements retrieved from the AXIS 1       */
/*           sensors which are mounted in the nadar and look forward of the   */
/*           spacecraft.  Every-other AXIS 1 sensor (or pixel) is stagered.   */
/*           AXIS 1 units cycle every 8.192 sec.  See Winningham et al., J.   */
/*           Geophys. Res., 10649-10666, 98, 1993 for a description of the    */
/*           instrument.                                                      */
/*                                                                            */
/*  AXIS1 binary files have two types of records.  The first is a header      */
/*  record which occurs only at the beginning of the file.  After this header */
/*  record comes a variable number of data records.                           */
/*                                                                            */
/*  Through out the AXIS1 binary files, values of -1.0e-31 is used as a fill  */
/*  value.  If this value occurs in a field, it means that that field is      */
/*  invalid.                                                                  */
/*                                                                            */
/*  Quality values should be zero.  A value of quality which is non-zero      */
/*  means that the entire sensor data is not valid, no matter what values     */
/*  occur within the data fields.  The quality values are different           */
/*  to indicate where failure occurred during processing and are used for     */
/*  diagnostic purposes only.                                                 */
/*                                                                            */
/*                                                                            */
/*  Global Structure Definitions:                                             */
/*                                                                            */
/*    Header Record Structure:                                                */
/*      float a1p1_eng[32];   32 values of AXIS 1 Pixel 1 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p2_eng[32];   32 values of AXIS 1 Pixel 2 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p3_eng[32];   32 values of AXIS 1 Pixel 3 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p4_eng[32];   32 values of AXIS 1 Pixel 4 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p5_eng[32];   32 values of AXIS 1 Pixel 5 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p6_eng[32];   32 values of AXIS 1 Pixel 6 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p7_eng[32];   32 values of AXIS 1 Pixel 7 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p8_eng[32];   32 values of AXIS 1 Pixel 8 sensor center       */
/*                            energy data in [eV]                             */
/*      float a1p1_ede[32];   32 values of AXIS 1 Pixel 1 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p2_ede[32];   32 values of AXIS 1 Pixel 2 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p3_ede[32];   32 values of AXIS 1 Pixel 3 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p4_ede[32];   32 values of AXIS 1 Pixel 4 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p5_ede[32];   32 values of AXIS 1 Pixel 5 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p6_ede[32];   32 values of AXIS 1 Pixel 6 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p7_ede[32];   32 values of AXIS 1 Pixel 7 sensor channel      */
/*                            width data in [eV]                              */
/*      float a1p8_ede[32];   32 values of AXIS 1 Pixel 8 sensor channel      */
/*                            width data in [eV]                              */
/*                                                                            */
/*    Data Record Structure:                                                  */
/*      long syear;          year the data started accumulating               */
/*      long sday;           day of year the data started accumulating        */
/*      long smsec;          millisecond of day the data started accumulating */
/*      long eyear;          year the data stopped accumulating               */
/*      long eday;           day of year the data stopped accumulating        */
/*      long emsec;          millisecond of day the data stopped accumulating */
/*      float cglat;         latitude at center of sweep [degrees]            */
/*      float cglon;         longitude at center of sweep [degrees]           */
/*      float cgalt;         altitude at center of sweep [km]                 */
/*      float cil600;        invariant latitude determined at center          */
/*                           of sweep for 600 km [degrees]                    */
/*      float cmst600;       magnetic solar time determined at center         */
/*                           of sweep for 600 km [degrees]                    */
/*      float csza;          solar zenith angle at center of sweep [degrees]  */
/*      float lshell;        L-shell of spacecraft [unitless]                 */
/*      float cpa[8];       8 values of pitch angle at center of sweep, one   */
/*                          for each pixel [degrees] - assignment:            */
/*                           cpa[0] = AXIS 1 PIXEL 1                          */
/*                           cpa[1] = AXIS 1 PIXEL 2                          */
/*                           cpa[2] = AXIS 1 PIXEL 3                          */
/*                           cpa[3] = AXIS 1 PIXEL 4                          */
/*                           cpa[4] = AXIS 1 PIXEL 5                          */
/*                           cpa[5] = AXIS 1 PIXEL 6                          */
/*                           cpa[6] = AXIS 1 PIXEL 7                          */
/*                           cpa[7] = AXIS 1 PIXEL 8                          */
/*      float temperatures[9];  9 values of different instrument temperatures */
/*                              affecting AXIS 1 [degC] - assignment:         */
/*                               temperatures[0] = AXIS 1 Preamp              */
/*                               temperatures[1] = AXIS 1 T-Beam, Cold        */
/*                               temperatures[2] = AXIS 1 T-Beam, Warm        */
/*                               temperatures[3] = AXIS Electronics Box       */
/*                               temperatures[4] = AXIS Base Plate            */
/*                               temperatures[5] = AXIS Housing               */
/*                               temperatures[6] = Radiator Primary, Cold     */
/*                               temperatures[7] = Radiator Primary, Warm     */
/*                               temperatures[8] = Radiator Secondary         */
/*      float a1p1_dnf[32];   32 values of AXIS 1 Pixel 1 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p2_dnf[32];   32 values of AXIS 1 Pixel 2 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p3_dnf[32];   32 values of AXIS 1 Pixel 3 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p4_dnf[32];   32 values of AXIS 1 Pixel 4 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p5_dnf[32];   32 values of AXIS 1 Pixel 5 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p6_dnf[32];   32 values of AXIS 1 Pixel 6 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p7_dnf[32];   32 values of AXIS 1 Pixel 7 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p8_dnf[32];   32 values of AXIS 1 Pixel 8 sensor differential */
/*                            number flux or intensity [(cm**2 sr s eV)**-1]  */
/*      float a1p1_cps[32];   32 values of AXIS 1 Pixel 1 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p2_cps[32];   32 values of AXIS 1 Pixel 2 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p3_cps[32];   32 values of AXIS 1 Pixel 3 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p4_cps[32];   32 values of AXIS 1 Pixel 4 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p5_cps[32];   32 values of AXIS 1 Pixel 5 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p6_cps[32];   32 values of AXIS 1 Pixel 6 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p7_cps[32];   32 values of AXIS 1 Pixel 7 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p8_cps[32];   32 values of AXIS 1 Pixel 8 sensor count rate   */
/*                            [counts/s]                                      */
/*      float a1p1_he1[32];   32 values of AXIS 1 Pixel 1 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p2_he1[32];   32 values of AXIS 1 Pixel 2 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p3_he1[32];   32 values of AXIS 1 Pixel 3 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p4_he1[32];   32 values of AXIS 1 Pixel 4 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p5_he1[32];   32 values of AXIS 1 Pixel 5 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p6_he1[32];   32 values of AXIS 1 Pixel 6 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p7_he1[32];   32 values of AXIS 1 Pixel 7 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p8_he1[32];   32 values of AXIS 1 Pixel 8 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 1 [counts/s]                       */
/*      float a1p1_he2[32];   32 values of AXIS 1 Pixel 1 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p2_he2[32];   32 values of AXIS 1 Pixel 2 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p3_he2[32];   32 values of AXIS 1 Pixel 3 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p4_he2[32];   32 values of AXIS 1 Pixel 4 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p5_he2[32];   32 values of AXIS 1 Pixel 5 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p6_he2[32];   32 values of AXIS 1 Pixel 6 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p7_he2[32];   32 values of AXIS 1 Pixel 7 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p8_he2[32];   32 values of AXIS 1 Pixel 8 predicted count     */
/*                            rate of high energy particle contamination      */
/*                            using method 2 [counts/s]                       */
/*      float a1p1_gcr1[32];  32 values of AXIS 1 Pixel 1 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p2_gcr1[32];  32 values of AXIS 1 Pixel 2 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p3_gcr1[32];  32 values of AXIS 1 Pixel 3 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p4_gcr1[32];  32 values of AXIS 1 Pixel 4 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p5_gcr1[32];  32 values of AXIS 1 Pixel 5 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p6_gcr1[32];  32 values of AXIS 1 Pixel 6 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p7_gcr1[32];  32 values of AXIS 1 Pixel 7 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p8_gcr1[32];  32 values of AXIS 1 Pixel 8 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 1 [counts/s]                       */
/*      float a1p1_gcr2[32];  32 values of AXIS 1 Pixel 1 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p2_gcr2[32];  32 values of AXIS 1 Pixel 2 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p3_gcr2[32];  32 values of AXIS 1 Pixel 3 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p4_gcr2[32];  32 values of AXIS 1 Pixel 4 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p5_gcr2[32];  32 values of AXIS 1 Pixel 5 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p6_gcr2[32];  32 values of AXIS 1 Pixel 6 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p7_gcr2[32];  32 values of AXIS 1 Pixel 7 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p8_gcr2[32];  32 values of AXIS 1 Pixel 8 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 2 [counts/s]                       */
/*      float a1p1_gcr3[32];  32 values of AXIS 1 Pixel 1 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p2_gcr3[32];  32 values of AXIS 1 Pixel 2 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p3_gcr3[32];  32 values of AXIS 1 Pixel 3 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p4_gcr3[32];  32 values of AXIS 1 Pixel 4 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p5_gcr3[32];  32 values of AXIS 1 Pixel 5 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p6_gcr3[32];  32 values of AXIS 1 Pixel 6 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p7_gcr3[32];  32 values of AXIS 1 Pixel 7 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p8_gcr3[32];  32 values of AXIS 1 Pixel 8 predicted count     */
/*                            rate of galactic cosmic ray contamination       */
/*                            using method 3 [counts/s]                       */
/*      float a1p1_plat[3];   3 values of AXIS 1 Pixel 1 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p1_plat[0] = latitude of inner edge          */
/*                             a1p1_plat[1] = latitude of pixel center        */
/*                             a1p1_plat[2] = latitude of outer edge          */
/*      float a1p2_plat[3];   3 values of AXIS 1 Pixel 2 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p2_plat[0] = latitude of inner edge          */
/*                             a1p2_plat[1] = latitude of pixel center        */
/*                             a1p2_plat[2] = latitude of outer edge          */
/*      float a1p3_plat[3];   3 values of AXIS 1 Pixel 3 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p3_plat[0] = latitude of inner edge          */
/*                             a1p3_plat[1] = latitude of pixel center        */
/*                             a1p3_plat[2] = latitude of outer edge          */
/*      float a1p4_plat[3];   3 values of AXIS 1 Pixel 4 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p4_plat[0] = latitude of inner edge          */
/*                             a1p4_plat[1] = latitude of pixel center        */
/*                             a1p4_plat[2] = latitude of outer edge          */
/*      float a1p5_plat[3];   3 values of AXIS 1 Pixel 5 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p5_plat[0] = latitude of inner edge          */
/*                             a1p5_plat[1] = latitude of pixel center        */
/*                             a1p5_plat[2] = latitude of outer edge          */
/*      float a1p6_plat[3];   3 values of AXIS 1 Pixel 6 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p6_plat[0] = latitude of inner edge          */
/*                             a1p6_plat[1] = latitude of pixel center        */
/*                             a1p6_plat[2] = latitude of outer edge          */
/*      float a1p7_plat[3];   3 values of AXIS 1 Pixel 7 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p7_plat[0] = latitude of inner edge          */
/*                             a1p7_plat[1] = latitude of pixel center        */
/*                             a1p7_plat[2] = latitude of outer edge          */
/*      float a1p8_plat[3];   3 values of AXIS 1 Pixel 8 projected latitude   */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p8_plat[0] = latitude of inner edge          */
/*                             a1p8_plat[1] = latitude of pixel center        */
/*                             a1p8_plat[2] = latitude of outer edge          */
/*      float a1p1_plon[3];   3 values of AXIS 1 Pixel 1 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p1_plon[0] = longitude of inner edge         */
/*                             a1p1_plon[1] = longitude of pixel center       */
/*                             a1p1_plon[2] = longitude of outer edge         */
/*      float a1p2_plon[3];   3 values of AXIS 1 Pixel 2 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p2_plon[0] = longitude of inner edge         */
/*                             a1p2_plon[1] = longitude of pixel center       */
/*                             a1p2_plon[2] = longitude of outer edge         */
/*      float a1p3_plon[3];   3 values of AXIS 1 Pixel 3 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p3_plon[0] = longitude of inner edge         */
/*                             a1p3_plon[1] = longitude of pixel center       */
/*                             a1p3_plon[2] = longitude of outer edge         */
/*      float a1p4_plon[3];   3 values of AXIS 1 Pixel 4 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p4_plon[0] = longitude of inner edge         */
/*                             a1p4_plon[1] = longitude of pixel center       */
/*                             a1p4_plon[2] = longitude of outer edge         */
/*      float a1p5_plon[3];   3 values of AXIS 1 Pixel 5 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p5_plon[0] = longitude of inner edge         */
/*                             a1p5_plon[1] = longitude of pixel center       */
/*                             a1p5_plon[2] = longitude of outer edge         */
/*      float a1p6_plon[3];   3 values of AXIS 1 Pixel 6 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p6_plon[0] = longitude of inner edge         */
/*                             a1p6_plon[1] = longitude of pixel center       */
/*                             a1p6_plon[2] = longitude of outer edge         */
/*      float a1p7_plon[3];   3 values of AXIS 1 Pixel 7 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p7_plon[0] = longitude of inner edge         */
/*                             a1p7_plon[1] = longitude of pixel center       */
/*                             a1p7_plon[2] = longitude of outer edge         */
/*      float a1p8_plon[3];   3 values of AXIS 1 Pixel 8 projected longitude  */
/*                            on the Earth at 100 km altitude [deg] -         */
/*                            assignment:                                     */
/*                             a1p8_plon[0] = longitude of inner edge         */
/*                             a1p8_plon[1] = longitude of pixel center       */
/*                             a1p8_plon[2] = longitude of outer edge         */
/*      unsigned char a1q[8];  8 values of AXIS 1 sensor quality assignment:  */
/*                               a1q[0] = AXIS 1 PIXEL 1                      */
/*                               a1q[1] = AXIS 1 PIXEL 2                      */
/*                               a1q[2] = AXIS 1 PIXEL 3                      */
/*                               a1q[3] = AXIS 1 PIXEL 4                      */
/*                               a1q[4] = AXIS 1 PIXEL 5                      */
/*                               a1q[5] = AXIS 1 PIXEL 6                      */
/*                               a1q[6] = AXIS 1 PIXEL 7                      */
/*                               a1q[7] = AXIS 1 PIXEL 8                      */
/*                               Note: values not equal to zero indicate bad  */
/*                                     and the data from that sensor should   */
/*                                     not be used                            */
/*      unsigned char a1p1_raw[32];  32 values of AXIS 1 PIXEL 1 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p2_raw[32];  32 values of AXIS 1 PIXEL 2 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p3_raw[32];  32 values of AXIS 1 PIXEL 3 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p4_raw[32];  32 values of AXIS 1 PIXEL 4 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p5_raw[32];  32 values of AXIS 1 PIXEL 5 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p6_raw[32];  32 values of AXIS 1 PIXEL 6 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p7_raw[32];  32 values of AXIS 1 PIXEL 7 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*      unsigned char a1p8_raw[32];  32 values of AXIS 1 PIXEL 8 raw          */
/*                                   (compressed) telemetry value [unitless]  */
/*                                                                            */
/*                                                                            */
/*  Global Variable Definitions:                                              */
/*                                                                            */
/*    int file_ptr;          pointer to the binary file (<0 means failure)    */
/*    int empty;             number of binary bites actually read (<0 means   */
/*                           failure)                                         */
/*                                                                            */
/*  Local Variable Definitions:                                               */
/*                                                                            */
/*    int argc;              number of command line arguments                 */
/*    char **argv;           pointer to a character string containing the     */
/*                           command line arguments                           */
/*    short btime_yr;        year of data file                                */
/*    short btime_day;       day of year of data file                         */
/*    short hr;              value of hour of day                             */
/*    short mn;              value of minute of hour                          */
/*    short sc;              value of second of minute                        */
/*    long spd;              number of seconds per day                        */
/*    float lat;             current latitude value                           */
/*    float lon;             current longitude value                          */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

/* 4567 10 234567 20 234567 30 234567 40 234567 50 234567 60 234567 70 234567 */
/******************************************************************************/
/* Header Record stores the Center Energies and Energy Widths for each AXIS   */
/* Pixel.                                                                     */
/******************************************************************************/

struct HEAD {
  float a1p1_eng[32];
  float a1p2_eng[32];
  float a1p3_eng[32];
  float a1p4_eng[32];
  float a1p5_eng[32];
  float a1p6_eng[32];
  float a1p7_eng[32];
  float a1p8_eng[32];
  float a1p1_ede[32];
  float a1p2_ede[32];
  float a1p3_ede[32];
  float a1p4_ede[32];
  float a1p5_ede[32];
  float a1p6_ede[32];
  float a1p7_ede[32];
  float a1p8_ede[32];
};

/******************************************************************************/
/* Data Record stores the Sweep Start and Stop times, Center O/A Values, the  */
/* Center Pitch Angle Value of the Sensor, instrument temperatures,           */
/* differential number flux (or intensity) Value, sensor count rate,          */
/* predicted count rate of high-energy particle contamination using method 1, */
/* predicted count rate of high-energy particle contamination using method 2, */
/* predicted count rate of galactic cosmic ray contamination using method 1,  */
/* predicted count rate of galactic cosmic ray contamination using method 2,  */
/* predicted count rate of galactic cosmic ray contamination using method 3,  */
/* projected pixel latitudes on the Earth's 100 km surface, projected pixel   */
/* longitudes on the Earth's 100 km surface, Sensor quality indicator, and    */
/* the RAW sensor count.                                                      */
/******************************************************************************/

struct DATA {
  long syear;
  long sday;
  long smsec;
  long eyear;
  long eday;
  long emsec;
  float cglat;
  float cglon;
  float cgalt;
  float cil600;
  float cmst600;
  float csza;
  float lshell;
  float cpa[8];
  float temperatures[9];
  float a1p1_dnf[32];
  float a1p2_dnf[32];
  float a1p3_dnf[32];
  float a1p4_dnf[32];
  float a1p5_dnf[32];
  float a1p6_dnf[32];
  float a1p7_dnf[32];
  float a1p8_dnf[32];
  float a1p1_cps[32];
  float a1p2_cps[32];
  float a1p3_cps[32];
  float a1p4_cps[32];
  float a1p5_cps[32];
  float a1p6_cps[32];
  float a1p7_cps[32];
  float a1p8_cps[32];
  float a1p1_he1[32];
  float a1p2_he1[32];
  float a1p3_he1[32];
  float a1p4_he1[32];
  float a1p5_he1[32];
  float a1p6_he1[32];
  float a1p7_he1[32];
  float a1p8_he1[32];
  float a1p1_he2[32];
  float a1p2_he2[32];
  float a1p3_he2[32];
  float a1p4_he2[32];
  float a1p5_he2[32];
  float a1p6_he2[32];
  float a1p7_he2[32];
  float a1p8_he2[32];
  float a1p1_gcr1[32];
  float a1p2_gcr1[32];
  float a1p3_gcr1[32];
  float a1p4_gcr1[32];
  float a1p5_gcr1[32];
  float a1p6_gcr1[32];
  float a1p7_gcr1[32];
  float a1p8_gcr1[32];
  float a1p1_gcr2[32];
  float a1p2_gcr2[32];
  float a1p3_gcr2[32];
  float a1p4_gcr2[32];
  float a1p5_gcr2[32];
  float a1p6_gcr2[32];
  float a1p7_gcr2[32];
  float a1p8_gcr2[32];
  float a1p1_gcr3[32];
  float a1p2_gcr3[32];
  float a1p3_gcr3[32];
  float a1p4_gcr3[32];
  float a1p5_gcr3[32];
  float a1p6_gcr3[32];
  float a1p7_gcr3[32];
  float a1p8_gcr3[32];
  float a1p1_plat[3];
  float a1p2_plat[3];
  float a1p3_plat[3];
  float a1p4_plat[3];
  float a1p5_plat[3];
  float a1p6_plat[3];
  float a1p7_plat[3];
  float a1p8_plat[3];
  float a1p1_plon[3];
  float a1p2_plon[3];
  float a1p3_plon[3];
  float a1p4_plon[3];
  float a1p5_plon[3];
  float a1p6_plon[3];
  float a1p7_plon[3];
  float a1p8_plon[3];
  unsigned char a1q[8];
  unsigned char a1p1_raw[32];
  unsigned char a1p2_raw[32];
  unsigned char a1p3_raw[32];
  unsigned char a1p4_raw[32];
  unsigned char a1p5_raw[32];
  unsigned char a1p6_raw[32];
  unsigned char a1p7_raw[32];
  unsigned char a1p8_raw[32];
};

/******************************************************************************/
/*  Global Variable Definitions are binary file indicator variables only.     */
/******************************************************************************/

int file_ptr, empty;

int main(int argc, char **argv)
{
  float lat, lon;
  long spd;
  int j,k;
  short btime_yr, btime_day;
  short hr, mn, sc;
  char bfilename[80], buf[80];
  struct HEAD header_rec;
  struct DATA data_rec;

  /****************************************************************************/
  /* Check for the proper number of command line arguments, output message if */
  /* not correct and quit the program.  If you have the correct number of     */
  /* inputs, the first is the year and the second is the day...get them.      */
  /****************************************************************************/

  if (argc != 3)
   {
      printf ("Calling sequence is: read_axis1_binary YEAR DAY\n");
      printf ("...try again\n");
      exit (-1);
   }
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_yr);
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_day);

  /****************************************************************************/
  /* Formulate the correct file name and then open that file.  Output error   */
  /* message and terminate if there is a problem.                             */
  /****************************************************************************/

  sprintf(bfilename,"PEM_AXIS1_%04d%03d_V01.DAT",btime_yr,btime_day);
  file_ptr = open(bfilename, O_RDONLY,0);
  if (file_ptr == -1)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not open file for output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    exit(-1);
  }

  /****************************************************************************/
  /* Read the header record.  Output error message and terminate if there is  */
  /* a problem.                                                               */
  /****************************************************************************/

  if ((empty=read(file_ptr,(char*)&header_rec,sizeof(header_rec))) < 0)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not read header record in output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    close(file_ptr);
    exit(-2);
  }

  /****************************************************************************/
  /* Write out Definitions of Column Headings.                                */
  /****************************************************************************/

  printf("Program Definitions and Units -\n");
  printf("     E = Center Energy of Channel [eV]\n");
  printf("    dE = Energy Channel Width [eV]\n");
  printf("   DNF = Differential Number Flux (or Intensity) ");
  printf("[(cm**2-s-sr-eV)**-1]\n");
  printf("   RAW = Raw Compressed Count Rate ");
  printf("in Telemetry [unitless]\n");
  printf("   CPS = Count Rate [counts/sec]\n");
  printf("   HE1 = High-Energy Particle Contamination Method 1 Count Rate ");
  printf("[counts/sec]\n");
  printf("   HE2 = High-Energy Particle Contamination Method 2 Count Rate ");
  printf("[counts/sec]\n");
  printf("  GCR1 = Galactic Cosmic Rays Contamination Method 1 Count Rate ");
  printf("[counts/sec]\n");
  printf("  GCR2 = Galactic Cosmic Rays Contamination Method 2 Count Rate ");
  printf("[counts/sec]\n");
  printf("  GCR3 = Galactic Cosmic Rays Contamination Method 3 Count Rate ");
  printf("[counts/sec]\n");
  printf("\n");
  printf("Notes -\n");
  printf("     1)  -1.0e+31 is used to indicate filled value\n");
  printf("     2)   1.0e+31 is used to indicate excluded value\n");
  printf("     3)  values less than -1.0e+31, between -1.0e-31 and +1.0e-31, ");
  printf("and greater than +1.0e+31\n");
  printf("         are used as fill values at different times during the ");
  printf("processing of telemetry\n");
  printf("     4)  non-zero quality values are for diagnostic purposes only ");
  printf("and indicate processing failure point\n");
  printf("\n");

  /****************************************************************************/
  /* Loop through reading the data records and read them.  terminate the      */
  /* program when the all have been read.                                     */
  /****************************************************************************/

  while ((empty=read(file_ptr,(char*)&data_rec,sizeof(data_rec))) > 0)
  {

    /**************************************************************************/
    /* For the Starting sweep time, convert the millisec per day into hours,  */
    /* minutes and seconds.  Print all starting sweep time values.            */
    /**************************************************************************/

    spd = data_rec.smsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Starting at - %04ld%03ld/%02d:%02d:%02d(%ld msec)\n",
            data_rec.syear,data_rec.sday,hr,mn,sc,data_rec.smsec);

    /**************************************************************************/
    /* For the Ending sweep time, convert the millisec per day into hours,    */
    /* minutes and seconds.  Print all ending sweep time values.              */
    /**************************************************************************/

    spd = data_rec.emsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Ending at - %04ld%03ld/%02d:%02d:%02d(%ld msec)\n",
            data_rec.eyear,data_rec.eday,hr,mn,sc,data_rec.emsec);

    /**************************************************************************/
    /* Some O/A parameters are given at the center time of the sweep.  Print  */
    /* these values.                                                          */
    /**************************************************************************/

    printf("Center Sweep Parameters ->\n");
    if (data_rec.cgalt < 0.0 || data_rec.cgalt > 1000.0)
                              printf("     Spacecraft Altitude = BAD,        ");
    else printf("     Spacecraft Altitude = %6.2f km,  ",data_rec.cgalt);
    if (data_rec.csza < 0.0 || data_rec.csza > 360.0)
                                           printf("Solar Zenith Angle = BAD\n");
    else printf("Solar Zenith Angle = %7.2f deg\n",data_rec.csza);
    if (data_rec.cglat < -90.0 || data_rec.cglat > 90.0)
                               printf("     Geodetic Latitude = BAD,         ");
    else printf("     Geodetic Latitude = %6.2f deg,  ",data_rec.cglat);
    if (data_rec.cglon < 0.0 || data_rec.cglon > 360.0)
                                               printf("East Longitude = BAD\n");
    else printf("East Longitude = %7.2f deg\n",data_rec.cglon);
    if (data_rec.cil600 < -90.0 || data_rec.cil600 > 90.0)
                    printf("     At 600 km: Invariant Latitude = BAD,         ");
    else printf("     At 600 km: Invariant Latitude = %6.2f deg,  ",
                                                               data_rec.cil600);
    if (data_rec.cmst600 < 0.0 || data_rec.cmst600 > 24.0)
                                          printf("Magnetic Solar Time = BAD\n");
    else printf("Magnetic Solar Time = %6.2f hr\n",data_rec.cmst600);
    if (data_rec.lshell < 0.0 || data_rec.lshell > 9999.0)
                                                 printf("     L-Shell = BAD\n");
    else printf("     L-Shell = %6.2f\n", data_rec.lshell);
    printf ("\n");

    /**************************************************************************/
    /* Print out the temperatures for this AXIS unit.                         */
    /**************************************************************************/

    printf("Instrument Temperatures ->\n");
    printf("     AXIS-1 PreAmp = ");
    if (data_rec.temperatures[0] > -1000.0 && data_rec.temperatures[0] < 1000.0)
              printf("%6.2f degC,", data_rec.temperatures[0]);
    else printf("BAD,       ");
    printf("                AXIS-1 T-Beam, Cold = ");
    if (data_rec.temperatures[1] > -1000.0 && data_rec.temperatures[1] < 1000.0)
              printf("%6.2f degC\n", data_rec.temperatures[1]);
    else printf("BAD\n");
    printf("     AXIS-1 T-Beam, Warm = ");
    if (data_rec.temperatures[2] > -1000.0 && data_rec.temperatures[2] < 1000.0)
              printf("%6.2f degC,", data_rec.temperatures[2]);
    else printf("BAD,       ");
    printf("          AXIS Electronic Box = ");
    if (data_rec.temperatures[3] > -1000.0 && data_rec.temperatures[3] < 1000.0)
              printf("%6.2f degC\n", data_rec.temperatures[3]);
    else printf("BAD\n");
    printf("     AXIS Base Plate = ");
    if (data_rec.temperatures[4] > -1000.0 && data_rec.temperatures[4] < 1000.0)
              printf("%6.2f degC,", data_rec.temperatures[4]);
    else printf("BAD,       ");
    printf("              AXIS Housing = ");
    if (data_rec.temperatures[5] > -1000.0 && data_rec.temperatures[5] < 1000.0)
              printf("%6.2f degC\n", data_rec.temperatures[5]);
    else printf("BAD\n");
    printf("     Radiator Primary, Cold = ");
    if (data_rec.temperatures[6] > -1000.0 && data_rec.temperatures[6] < 1000.0)
              printf("%6.2f degC,", data_rec.temperatures[6]);
    else printf("BAD,        ");
    printf("      Radiator Primary, Warm = ");
    if (data_rec.temperatures[7] > -1000.0 && data_rec.temperatures[7] < 1000.0)
              printf("%6.2f degC\n", data_rec.temperatures[7]);
    else printf("BAD\n");
    printf("     Radiator Secondary = ");
    if (data_rec.temperatures[8] > -1000.0 && data_rec.temperatures[8] < 1000.0)
              printf("%6.2f degC\n", data_rec.temperatures[8]);
    else printf("BAD\n");
    printf ("\n");

    /**************************************************************************/
    /* Go through each AXIS Pixel at a time.  Decipher and print the result.  */
    /**************************************************************************/

    for (j=0; j<8; ++j)
    {

      /************************************************************************/
      /* Print the AXIS unit and the Pixel.  Print each pixel's quality and   */
      /* pitch angle at the center of the acquisition.  If the quality is not */
      /* 0, a error during processing was indicated.  If the quality is       */
      /* non-zero, the word "BAD" is printed.  The range of the pitch angle   */
      /* is from 0 deg to 180 deg.  Any number out side of this range means   */
      /* the quality is not valid and the word "BAD" is printed.              */
      /************************************************************************/

      printf("AXIS 1 Pixel %01d: ",j+1);
      printf("  QUALITY ");
      if (((int) data_rec.a1q[j]) != 0) printf("BAD  ");
      else printf("%03d  ",(int) data_rec.a1q[j]);
      printf("                    ");
      printf(" Center Pitch Angle: ");
      if (data_rec.cpa[j] >= 0.0 && data_rec.cpa[j] <= 180.0) 
               printf(" %8.3f deg",data_rec.cpa[j]);
      else printf("BAD ");
      printf ("\n");

      /************************************************************************/
      /* Each pixel is projected on the 100 km surface of the Earth.  For     */
      /* each pixel, look up this value and print the edges and center        */
      /* latitude and longitude pairs.                                        */
      /************************************************************************/

      for (k=0; k<3; ++k)
      {
        if (j == 0)
        {
          lat = data_rec.a1p1_plat[k];
          lon = data_rec.a1p1_plon[k];
        }
        else if (j == 1)
        {
          lat = data_rec.a1p2_plat[k];
          lon = data_rec.a1p2_plon[k];
        }
        else if (j == 2)
        {
          lat = data_rec.a1p3_plat[k];
          lon = data_rec.a1p3_plon[k];
        }
        else if (j == 3)
        {
          lat = data_rec.a1p4_plat[k];
          lon = data_rec.a1p4_plon[k];
        }
        else if (j == 4)
        {
          lat = data_rec.a1p5_plat[k];
          lon = data_rec.a1p5_plon[k];
        }
        else if (j == 5)
        {
          lat = data_rec.a1p6_plat[k];
          lon = data_rec.a1p6_plon[k];
        }
        else if (j == 6)
        {
          lat = data_rec.a1p7_plat[k];
          lon = data_rec.a1p7_plon[k];
        }
        else
        {
          lat = data_rec.a1p8_plat[k];
          lon = data_rec.a1p8_plon[k];
        }
        printf("                  ");
        printf("Latitude (");
        if (k == 0) printf("out");
        if (k == 1) printf("center");
        if (k == 2) printf("in");
        if (lat > -90.0 && lat < 90.0) printf ("): %6.2f deg", lat);
        else printf ("): BAD       ");
        if (k == 0) printf("   ");
        if (k == 2) printf("    ");
        printf("     ");
        printf ("Longitude (");
        if (k == 0) printf("out");
        if (k == 1) printf("center");
        if (k == 2) printf("in");
        if (lon > -360.0 && lon < 360.0) printf ("): %7.2f deg\n", lon);
        else printf ("): BAD\n");
      }
      printf ("\n");

      /************************************************************************/
      /* If the quality of the AXIS sensor is bad, don't print out any        */
      /* values, just get the next Pixel's data.                              */
      /************************************************************************/

      if (((int) data_rec.a1q[j]) != 0)
      {
        printf ("\n");
        continue;
      }

      /************************************************************************/
      /* Print headings for Pixel parameters.                                 */
      /************************************************************************/

      printf("    E         dE         DNF    (RAW)      CPS          HE1");
      printf("          HE2         GCR1         GCR2         GCR3\n");

      /************************************************************************/
      /* Specific values are printed depending on which Pixel's data are      */
      /* being examined.  They have to be handled separately.  The first      */
      /* considered is for AXIS-1, Pixel-1.                                   */
      /************************************************************************/

      if (j == 0)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p1_eng[k] > 0 && header_rec.a1p1_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p1_eng[k]);
          else printf("%9.1e ",header_rec.a1p1_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p1_ede[k] > 0 && header_rec.a1p1_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p1_ede[k]);
          else printf("%9.1e ",header_rec.a1p1_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p1_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p1_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p1_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p1_he1[k]);
          printf("%12.5e:",data_rec.a1p1_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p1_gcr1[k]);
          printf("%12.5e:",data_rec.a1p1_gcr2[k]);
          printf("%12.5e",data_rec.a1p1_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The second considered is for AXIS-1, Pixel-2.                        */
      /************************************************************************/

      if (j == 1)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p2_eng[k] > 0 && header_rec.a1p2_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p2_eng[k]);
          else printf("%9.1e ",header_rec.a1p2_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p2_ede[k] > 0 && header_rec.a1p2_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p2_ede[k]);
          else printf("%9.1e ",header_rec.a1p2_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p2_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p2_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p2_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p2_he1[k]);
          printf("%12.5e:",data_rec.a1p2_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p2_gcr1[k]);
          printf("%12.5e:",data_rec.a1p2_gcr2[k]);
          printf("%12.5e",data_rec.a1p2_gcr3[k]);
          printf ("\n");
          }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The third considered is for AXIS-1, Pixel-3.                         */
      /************************************************************************/

      if (j == 2)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p3_eng[k] > 0 && header_rec.a1p3_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p3_eng[k]);
          else printf("%9.1e ",header_rec.a1p3_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p3_ede[k] > 0 && header_rec.a1p3_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p3_ede[k]);
          else printf("%9.1e ",header_rec.a1p3_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p3_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p3_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p3_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p3_he1[k]);
          printf("%12.5e:",data_rec.a1p3_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p3_gcr1[k]);
          printf("%12.5e:",data_rec.a1p3_gcr2[k]);
          printf("%12.5e",data_rec.a1p3_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The fourth considered is for AXIS-1, Pixel-4.                        */
      /************************************************************************/

      if (j == 3)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p4_eng[k] > 0 && header_rec.a1p4_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p4_eng[k]);
          else printf("%9.1e ",header_rec.a1p4_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p4_ede[k] > 0 && header_rec.a1p4_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p4_ede[k]);
          else printf("%9.1e ",header_rec.a1p4_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p4_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p4_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p4_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p4_he1[k]);
          printf("%12.5e:",data_rec.a1p4_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p4_gcr1[k]);
          printf("%12.5e:",data_rec.a1p4_gcr2[k]);
          printf("%12.5e",data_rec.a1p4_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The fifth considered is for AXIS-1, Pixel-5.                         */
      /************************************************************************/

      if (j == 4)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p5_eng[k] > 0 && header_rec.a1p5_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p5_eng[k]);
          else printf("%9.1e ",header_rec.a1p5_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p5_ede[k] > 0 && header_rec.a1p5_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p5_ede[k]);
          else printf("%9.1e ",header_rec.a1p5_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p5_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p5_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p5_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p5_he1[k]);
          printf("%12.5e:",data_rec.a1p5_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p5_gcr1[k]);
          printf("%12.5e:",data_rec.a1p5_gcr2[k]);
          printf("%12.5e",data_rec.a1p5_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The sixth considered is for AXIS-1, Pixel-6.                         */
      /************************************************************************/

      if (j == 5)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p6_eng[k] > 0 && header_rec.a1p6_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p6_eng[k]);
          else printf("%9.1e ",header_rec.a1p6_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p6_ede[k] > 0 && header_rec.a1p6_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p6_ede[k]);
          else printf("%9.1e ",header_rec.a1p6_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p6_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p6_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p6_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p6_he1[k]);
          printf("%12.5e:",data_rec.a1p6_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p6_gcr1[k]);
          printf("%12.5e:",data_rec.a1p6_gcr2[k]);
          printf("%12.5e",data_rec.a1p6_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The seventh considered is for AXIS-1, Pixel-7.                       */
      /************************************************************************/

      if (j == 6)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p7_eng[k] > 0 && header_rec.a1p7_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p7_eng[k]);
          else printf("%9.1e ",header_rec.a1p7_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p7_ede[k] > 0 && header_rec.a1p7_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p7_ede[k]);
          else printf("%9.1e ",header_rec.a1p7_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p7_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p7_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p7_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p7_he1[k]);
          printf("%12.5e:",data_rec.a1p7_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p7_gcr1[k]);
          printf("%12.5e:",data_rec.a1p7_gcr2[k]);
          printf("%12.5e",data_rec.a1p7_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The eighth considered is for AXIS-1, Pixel-8.                        */
      /************************************************************************/

      if (j == 7)
      {

        /**********************************************************************/
        /* There are 32 data values for this Pixel, get ready to print them   */
        /* if needed.                                                         */
        /**********************************************************************/

        for (k=0; k<32; ++k)
        {

          /********************************************************************/
          /* Print the channel center energy [eV].  Print negative values     */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p8_eng[k] > 0 && header_rec.a1p8_eng[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p8_eng[k]);
          else printf("%9.1e ",header_rec.a1p8_eng[k]);

          /********************************************************************/
          /* Print the channel energy width [eV].  Print negative values      */
          /* with less precision.                                             */
          /********************************************************************/

          if (header_rec.a1p8_ede[k] > 0 && header_rec.a1p8_ede[k] < 1.0e7)
                   printf("%9.2e ",header_rec.a1p8_ede[k]);
          else printf("%9.1e ",header_rec.a1p8_ede[k]);

          /********************************************************************/
          /* Print the measured differential number flux (or intensity) for   */
          /* the pixel [(cm**2-sr-s-eV)**-1].                                 */
          /********************************************************************/

          printf("%12.5e",data_rec.a1p8_dnf[k]);

          /********************************************************************/
          /* Print the RAW count for the pixel.  This is the compressed       */
          /* telemetry value [unitless].                                      */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.a1p8_raw[k]);

          /********************************************************************/
          /* Print the pixel count rate [counts/s].                           */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p8_cps[k]);

          /********************************************************************/
          /* There are two methods for predicting the count rate from         */
          /* high-energy particle contamination.  Method 1 normalized an      */
          /* empirically generated universal spectrum created from            */
          /* high-energy protons and electrons using the count rate found in  */
          /* energy channels 19-21.  Method 2 uses the same universal         */
          /* spectrum, but normalizes with the count rate found in energy     */
          /* channels 28-28.  Print these two predicted spectra.              */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p8_he1[k]);
          printf("%12.5e:",data_rec.a1p8_he2[k]);

          /********************************************************************/
          /* There are three methods for predicting the count rate from       */
          /* galactic cosmic rays.  Method 1 uses a standard spectrum for     */
          /* galactic cosmic rays that is normalized by the continuous        */
          /* L-shell value.  Method 2 uses the same spectrum and              */
          /* normalization except the square root of the result of Method 1   */
          /* is added to method 2.  Method 3 is the same as Method 1 except   */
          /* that the L-shell is only determined with the resolution of 1/2   */
          /* L.  Print these three predicted spectra.                         */
          /********************************************************************/

          printf("%12.5e:",data_rec.a1p8_gcr1[k]);
          printf("%12.5e:",data_rec.a1p8_gcr2[k]);
          printf("%12.5e",data_rec.a1p8_gcr3[k]);
          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 32 elements for the AXIS Pixel.                        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Get the next AXIS Pixel to print.                                    */
      /************************************************************************/

      printf ("\n");
    }

    /**************************************************************************/
    /* Looked at the data from all of the AXIS Pixels stored in this record;  */
    /* need another data record.                                              */
    /**************************************************************************/

    printf ("\n");
  }

  /****************************************************************************/
  /* End to data in that file.                                                */
  /****************************************************************************/

  close(file_ptr);
  printf("\nEOF\n");
}
