/******************************************************************************/
/*                                                                            */
/*  Version: 2.00                                                             */
/*  Date: October 30, 2002                                                    */
/*  Code Written By: Rudy Frahm                                               */
/*                   Southwest Research Institute                             */
/*                   PO Drawer 28510                                          */
/*                   San Antonio, TX  78228                                   */
/*                   Office: (210) 522-3855                                   */
/*                   FAX: (210) 647-4325                                      */
/*                   E-mail: rudy@pemrac.space.swri.edu                       */
/*                                                                            */
/*  Purpose: This program reads the HEPSA binary files and prints them.       */
/*           HEPSA data contains measurements retrieved from the electron     */
/*           HEPS sensors which are mounted in the zenith and cycle every     */
/*           4.096 sec.  See Winningham et al., J. Geophys. Res.,             */
/*           10649-10666, 98, 1993 for a description of the instrument.       */
/*                                                                            */
/*  HEPSA binary files have two types of records.  The first is a header      */
/*  record which occurs only at the beginning of the file.  After this header */
/*  record comes a variable number of data records.                           */
/*                                                                            */
/*  Through out the HEPSA binary files, values of -1.0e-31 is used as a fill  */
/*  value.  If this value occurs in a field, it means that that field is      */
/*  invalid.                                                                  */
/*                                                                            */
/*  Quality values should be zero.  A value of quality which is non-zero      */
/*  means that the entire sensor data is not valid, mo matter what values     */
/*  occur within the data fields.  The quality values have different values   */
/*  to indicate where failure occurred during processing and are used for     */
/*  diagnostic purposes only.                                                 */
/*                                                                            */
/*                                                                            */
/*  Global Structure Definitions:                                             */
/*                                                                            */
/*    Header Record Structure:                                                */
/*      float eh1t1de_eng[16];   16 values of electron HEPS 1 Telescope 1 DE  */
/*                               sensor center energy data in [eV]            */
/*      float eh1t1ee_eng[16];   16 values of electron HEPS 1 Telescope 1 EE  */
/*                               sensor center energy data in [eV]            */
/*      float eh1t2de_eng[16];   16 values of electron HEPS 1 Telescope 2 DE  */
/*                               sensor center energy data in [eV]            */
/*      float eh1t2ee_eng[16];   16 values of electron HEPS 1 Telescope 2 EE  */
/*                               sensor center energy data in [eV]            */
/*      float eh2t1de_eng[16];   16 values of electron HEPS 2 Telescope 1 DE  */
/*                               sensor center energy data in [eV]            */
/*      float eh2t1ee_eng[16];   16 values of electron HEPS 2 Telescope 1 EE  */
/*                               sensor center energy data in [eV]            */
/*      float eh2t2de_eng[16];   16 values of electron HEPS 2 Telescope 2 DE  */
/*                               sensor center energy data in [eV]            */
/*      float eh2t2ee_eng[16];   16 values of electron HEPS 2 Telescope 2 EE  */
/*                               sensor center energy data in [eV]            */
/*      float eh1t1de_ede[16];   16 values of electron HEPS 1 Telescope 1 DE  */
/*                               sensor channel width data in [eV]            */
/*      float eh1t1ee_ede[16];   16 values of electron HEPS 1 Telescope 1 EE  */
/*                               sensor channel width data in [eV]            */
/*      float eh1t2de_ede[16];   16 values of electron HEPS 1 Telescope 2 DE  */
/*                               sensor channel width data in [eV]            */
/*      float eh1t2ee_ede[16];   16 values of electron HEPS 1 Telescope 2 EE  */
/*                               sensor channel width data in [eV]            */
/*      float eh2t1de_ede[16];   16 values of electron HEPS 2 Telescope 1 DE  */
/*                               sensor channel width data in [eV]            */
/*      float eh2t1ee_ede[16];   16 values of electron HEPS 2 Telescope 1 EE  */
/*                               sensor channel width data in [eV]            */
/*      float eh2t2de_ede[16];   16 values of electron HEPS 2 Telescope 2 DE  */
/*                               sensor channel width data in [eV]            */
/*      float eh2t2ee_ede[16];   16 values of electron HEPS 2 Telescope 2 EE  */
/*                               sensor channel width data in [eV]            */
/*      float h_err[256];    256 values consisting of a fraction of error     */
/*                           assigned to each HEPS telemetry value, including */
/*                           Poisson statistics, telemetry compression, and   */
/*                           instrument errors [unitless]                     */
/*                                                                            */
/*    Data Record Structure:                                                  */
/*      long syear;          year the data started accumulating               */
/*      long sday;           day of year the data started accumulating        */
/*      long smsec;          millisecond of day the data started accumulating */
/*      long eyear;          year the data stopped accumulating               */
/*      long eday;           day of year the data stopped accumulating        */
/*      long emsec;          millisecond of day the data stopped accumulating */
/*      float cglat;         latitude at center of sweep [degrees]            */
/*      float cglon;         longitude at center of sweep [degrees]           */
/*      float cgalt;         altitude at center of sweep [km]                 */
/*      float cil600;        invariant latitude determined at center          */
/*                           of sweep for 600 km [degrees]                    */
/*      float cmst600;       magnetic solar time determined at center         */
/*                           of sweep for 600 km [degrees]                    */
/*      float csza;          solar zenith angle at center of sweep [degrees]  */
/*      float cpa[8];        8 values of pitch angle at center of sweep       */
/*                           [degrees] - assignment (DE and EE crystals have  */
/*                                the same look directions):                  */
/*                           cpa[0] = HEPS 1 Telescope 1 DE                   */
/*                           cpa[1] = HEPS 1 Telescope 1 EE                   */
/*                           cpa[2] = HEPS 1 Telescope 2 DE                   */
/*                           cpa[3] = HEPS 1 Telescope 2 EE                   */
/*                           cpa[4] = HEPS 2 Telescope 1 DE                   */
/*                           cpa[5] = HEPS 2 Telescope 1 EE                   */
/*                           cpa[6] = HEPS 2 Telescope 2 DE                   */
/*                           cpa[7] = HEPS 2 Telescope 2 EE                   */
/*      float eh1t1de_dnf[16];   16 values of electron HEPS 1 Telescope 1 DE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh1t1ee_dnf[16];   16 values of electron HEPS 1 Telescope 1 EE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh1t2de_dnf[16];   16 values of electron HEPS 1 Telescope 2 DE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh1t2ee_dnf[16];   16 values of electron HEPS 1 Telescope 2 EE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh2t1de_dnf[16];   16 values of electron HEPS 2 Telescope 1 DE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh2t1ee_dnf[16];   16 values of electron HEPS 2 Telescope 1 EE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh2t2de_dnf[16];   16 values of electron HEPS 2 Telescope 2 DE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      float eh2t2ee_dnf[16];   16 values of electron HEPS 2 Telescope 2 EE  */
/*                               differential number flux or intensity        */
/*                               [(cm**2 sr s eV)**-1]                        */
/*      unsigned char eq[8];     8 values of electron sensor quality          */
/*                               assignment:                                  */
/*                               eq[0] = electron HEPS 1 Telescope 1 DE       */
/*                               eq[1] = electron HEPS 1 Telescope 1 EE       */
/*                               eq[2] = electron HEPS 1 Telescope 2 DE       */
/*                               eq[3] = electron HEPS 1 Telescope 2 EE       */
/*                               eq[4] = electron HEPS 2 Telescope 1 DE       */
/*                               eq[5] = electron HEPS 2 Telescope 1 EE       */
/*                               eq[6] = electron HEPS 2 Telescope 2 DE       */
/*                               eq[7] = electron HEPS 2 Telescope 2 EE       */
/*                               Note: values not equal to zero indicate bad  */
/*                                     and the data from that sensor should   */
/*                                     not be used                            */
/*      unsigned char eh1t1de_raw[16];                                        */
/*                               16 values of electron HEPS 1 Telescope 1 DE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh1t1ee_raw[16];                                        */
/*                               16 values of electron HEPS 1 Telescope 1 EE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh1t2de_raw[16];                                        */
/*                               16 values of electron HEPS 1 Telescope 2 DE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh1t2ee_raw[16];                                        */
/*                               16 values of electron HEPS 1 Telescope 2 EE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh2t1de_raw[16];                                        */
/*                               16 values of electron HEPS 2 Telescope 1 DE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh2t1ee_raw[16];                                        */
/*                               16 values of electron HEPS 2 Telescope 1 EE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh2t2de_raw[16];                                        */
/*                               16 values of electron HEPS 2 Telescope 2 DE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*      unsigned char eh2t2ee_raw[16];                                        */
/*                               16 values of electron HEPS 2 Telescope 2 EE  */
/*                               raw telemetry value (used in conjunction     */
/*                               with the error table stored in the header    */
/*                               record to determine the fractional error in  */
/*                               the data value)                              */
/*                                                                            */
/*                                                                            */
/*  Global Variable Definitions:                                              */
/*                                                                            */
/*    int file_ptr;          pointer to the binary file (<0 means failure)    */
/*    int empty;             number of binary bites actually read (<0 means   */
/*                           failure)                                         */
/*                                                                            */
/*  Local Variable Definitions:                                               */
/*                                                                            */
/*    int argc;              number of command line arguments                 */
/*    char **argv;           pointer to a character string containing the     */
/*                           command line arguments                           */
/*    short btime_yr;        year of data file                                */
/*    short btime_day;       day of year of data file                         */
/*    short hr;              value of hour of day                             */
/*    short mn;              value of minute of hour                          */
/*    short sc;              value of second of minute                        */
/*    long spd;              number of seconds per day                        */
/*                           command line arguments                           */
/*    int tel;               HEPS Telescope number (1 or 2)                   */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

/* 4567 10 234567 20 234567 30 234567 40 234567 50 234567 60 234567 70 234567 */
/******************************************************************************/
/* Header Record stores the Center Energies, Energy Widths, and the error     */
/* fraction table used to determine the fraction of the differential number   */
/* which is the standard deviation.                                           */
/******************************************************************************/

struct HEAD {
  float eh1t1de_eng[16];
  float eh1t1ee_eng[16];
  float eh1t2de_eng[16];
  float eh1t2ee_eng[16];
  float eh2t1de_eng[16];
  float eh2t1ee_eng[16];
  float eh2t2de_eng[16];
  float eh2t2ee_eng[16];
  float eh1t1de_ede[16];
  float eh1t1ee_ede[16];
  float eh1t2de_ede[16];
  float eh1t2ee_ede[16];
  float eh2t1de_ede[16];
  float eh2t1ee_ede[16];
  float eh2t2de_ede[16];
  float eh2t2ee_ede[16];
  float h_err[256];
};

/******************************************************************************/
/* Data Record stores the Sweep Start and Stop times, Center O/A Values, the  */
/* Center Pitch Angle Value of the Sensor, differential number flux (or       */
/* intensity) Value, Sensor quality indicator, and the RAW count (used as     */
/* an index for determining the error in the measurement).                    */
/******************************************************************************/

struct DATA {
  long syear;
  long sday;
  long smsec;
  long eyear;
  long eday;
  long emsec;
  float cglat;
  float cglon;
  float cgalt;
  float cil600;
  float cmst600;
  float csza;
  float cpa[8];
  float eh1t1de_dnf[16];
  float eh1t1ee_dnf[16];
  float eh1t2de_dnf[16];
  float eh1t2ee_dnf[16];
  float eh2t1de_dnf[16];
  float eh2t1ee_dnf[16];
  float eh2t2de_dnf[16];
  float eh2t2ee_dnf[16];
  unsigned char eq[8];
  unsigned char eh1t1de_raw[16];
  unsigned char eh1t1ee_raw[16];
  unsigned char eh1t2de_raw[16];
  unsigned char eh1t2ee_raw[16];
  unsigned char eh2t1de_raw[16];
  unsigned char eh2t1ee_raw[16];
  unsigned char eh2t2de_raw[16];
  unsigned char eh2t2ee_raw[16];
};

/******************************************************************************/
/*  Global Variable Definitions are binary file indicator variables only.     */
/******************************************************************************/

int file_ptr, empty;

void main(int argc, char **argv)
{
  long spd;
  int j,k,tel;
  short btime_yr, btime_day;
  short hr, mn, sc;
  char bfilename[80], buf[80];
  struct HEAD header_rec;
  struct DATA data_rec;

  /****************************************************************************/
  /* Check for the proper number of command line arguments, output message if */
  /* not correct and quit the program.  If you have the correct number of     */
  /* inputs, the first is the year and the second is the day...get them.      */
  /****************************************************************************/

  if (argc != 3)
   {
      printf ("Calling sequence is: read_hepsa_binary YEAR DAY\n");
      printf ("...try again\n");
      exit (-1);
   }
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_yr);
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_day);

  /****************************************************************************/
  /* Formulate the correct file name and then open that file.  Output error   */
  /* message and terminate if there is a problem.                             */
  /****************************************************************************/

  sprintf(bfilename,"PEM_HEPSA_%04d%03d_V02.DAT",btime_yr,btime_day);
  file_ptr = open(bfilename, O_RDONLY,0);
  if (file_ptr == -1)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not open file for output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    exit(-1);
  }

  /****************************************************************************/
  /* Read the header record.  Output error message and terminate if there is  */
  /* a problem.                                                               */
  /****************************************************************************/

  if ((empty=read(file_ptr,(char*)&header_rec,sizeof(header_rec))) < 0)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not read header record in output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    close(file_ptr);
    exit(-2);
  }

  /****************************************************************************/
  /* Write out Definitions of Column Headings.                                */
  /****************************************************************************/

  printf("Program Definitions and Units -\n");
  printf("     E = Center Energy of Channel [eV]\n");
  printf("    dE = Energy Channel Width [eV]\n");
  printf("   DNF = Differential Number Flux (or Intensity) ");
  printf("[(cm**2-s-sr-eV)**-1]\n");
  printf("   RAW = Raw Compressed Count Rate ");
  printf("in Telemetry (index for error) [unitless]\n");
  printf("  dDNF = Standard Deviation of DNF Measurement ");
  printf("[(cm**2-s-sr-eV)**-1]\n");
  printf("\n");
  printf("Notes -\n");
  printf("     1)  -1.0e+31 is used to indicate filled value\n");
  printf("     2)   1.0e+31 is used to indicate excluded value\n");
  printf("     3)  negative fluxes result from subtraction of temperature ");
  printf("dependent background spectra\n");
  printf("         and/or subtraction of expected contamination from protons\n");
  printf("     4)  non-zero quality values are for diagnostic purposes only ");
  printf("and indicate processing failure point\n");
  printf("\n");

  /****************************************************************************/
  /* Loop through reading the data records and read them.  terminate the      */
  /* program when the all have been read.                                     */
  /****************************************************************************/

  while ((empty=read(file_ptr,(char*)&data_rec,sizeof(data_rec))) > 0)
  {

    /**************************************************************************/
    /* For the Starting sweep time, convert the millisec per day into hours,  */
    /* minutes and seconds.  Print all starting sweep time values.            */
    /**************************************************************************/

    spd = data_rec.smsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Starting at - %04ld%03ld/%02d:%02d:%02d(%ld msec)\n",
            data_rec.syear,data_rec.sday,hr,mn,sc,data_rec.smsec);

    /**************************************************************************/
    /* For the Ending sweep time, convert the millisec per day into hours,    */
    /* minutes and seconds.  Print all ending sweep time values.              */
    /**************************************************************************/

    spd = data_rec.emsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Ending at - %04ld%03ld/%02d:%02d:%02d(%ld msec)\n",
            data_rec.eyear,data_rec.eday,hr,mn,sc,data_rec.emsec);

    /**************************************************************************/
    /* Some O/A parameters are given at the center time of the sweep.  Print  */
    /* these values.                                                          */
    /**************************************************************************/

    printf("Center Sweep Parameters ->\n");
    if (data_rec.cgalt < 0.0 || data_rec.cgalt > 1000.0)
                              printf("     Spacecraft Altitude = BAD,        ");
    else printf("     Spacecraft Altitude = %6.2f km,  ",data_rec.cgalt);
    if (data_rec.csza < 0.0 || data_rec.csza > 360.0)
                                           printf("Solar Zenith Angle = BAD\n");
    else printf("Solar Zenith Angle = %7.2f deg\n",data_rec.csza);
    if (data_rec.cglat < -90.0 || data_rec.cglat > 90.0)
                               printf("     Geodetic Latitude = BAD,         ");
    else printf("     Geodetic Latitude = %6.2f deg,  ",data_rec.cglat);
    if (data_rec.cglon < 0.0 || data_rec.cglon > 360.0)
                                               printf("East Longitude = BAD\n");
    else printf("East Longitude = %7.2f deg\n",data_rec.cglon);
    if (data_rec.cil600 < -90.0 || data_rec.cil600 > 90.0)
                    printf("     At 600 km: Invariant Latitude = BAD,         ");
    else printf("     At 600 km: Invariant Latitude = %6.2f deg,  ",
                                                               data_rec.cil600);
    if (data_rec.cmst600 < 0.0 || data_rec.cmst600 > 24.0)
                                          printf("Magnetic Solar Time = BAD\n");
    else printf("Magnetic Solar Time = %6.2f hr\n",data_rec.cmst600);
    printf ("\n");

    /**************************************************************************/
    /* Go through each sensors' data a telescope at a time.  Decipher and     */
    /* print the result.                                                      */
    /**************************************************************************/

    for (j=0; j<8; j+=2)
    {

      /************************************************************************/
      /* Determine the HEPS unit and telescope.  Handle data from the DE and  */
      /* EE parts of the telescope together.                                  */
      /************************************************************************/

      tel = 1;
      if (j == 2 || j == 3 || j >= 6) tel = 2;

      /************************************************************************/
      /* Print the HEPS unit and Telescope.  Print the pitch angle ate the    */
      /* center of the energy acquisition. The range of the pitch angle is    */
      /* from 0 deg to 180 deg.  Any number out side of this range means the  */
      /* quality is not valid and the word "BAD" is printed.                  */
      /************************************************************************/

      printf("HEPS %01d Telescope %01d             ",j/4+1,tel);
      printf(" Center Pitch Angle: ");
      if (data_rec.cpa[j+1] >= 0.0 && data_rec.cpa[j+1] <= 180.0) 
          printf(" %8.3f deg",data_rec.cpa[j+1]);
      else printf("BAD ");
      printf ("\n");

      /************************************************************************/
      /* The quality values of the EE and DE Electron sensors are evaluated.  */
      /* If they are not 0, a error during processing was indicated.  If the  */
      /* quality in non-zero, the word "BAD" is printed.                      */
      /************************************************************************/

      printf(" HEPS Electron EE Sensor:  QUALITY ");
      if (((int) data_rec.eq[j+1]) != 0) printf("BAD  ");
      else printf("%03d  ",(int) data_rec.eq[j+1]);
      printf("             ");
      printf("  HEPS Electron DE Sensor:  QUALITY ");
      if (((int) data_rec.eq[j]) != 0) printf("BAD  ");
      else printf("%03d  ",(int) data_rec.eq[j]);
      printf ("\n");

      /************************************************************************/
      /* If the quality of both the EE and DE sensors are bad, don't print    */
      /* out any values, just get the next Telescope's data.                  */
      /************************************************************************/

      if ((((int) data_rec.eq[j+1]) != 0) && (((int) data_rec.eq[j]) != 0)) 
      {
        printf ("\n");
        continue;
      }

      /************************************************************************/
      /* Print headings for EE parameters if the quality of that sensor is    */
      /* good.                                                                */
      /************************************************************************/

      if (((int) data_rec.eq[j+1]) != 0)
        printf("                                                   ");
      else
        printf("    E         dE         DNF    (RAW)     dDNF       ");

      /************************************************************************/
      /* Print headings for DE parameters if the quality of that sensor is    */
      /* good.                                                                */
      /************************************************************************/

      if (((int) data_rec.eq[j]) != 0)
        printf("                                                   ");
      else
        printf("    E         dE         DNF    (RAW)     dDNF     ");
      printf ("\n");

      /************************************************************************/
      /* Specific values are printed depending on which HEPS unit and         */
      /* Telescope's data are being examined.  They have to be handled        */
      /* separately.  The first considered is for HEPS-1, Telescope-1.        */
      /************************************************************************/

      if (j == 0)
      {
 
        /**********************************************************************/
        /* There are 16 data values for this telescope, get ready to print    */
        /* them if needed.                                                    */
        /**********************************************************************/

        for (k=0; k<16; ++k)
        {

          /********************************************************************/
          /* Check the quality for the EE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j+1]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh1t1ee_eng[k] < 500.0 
                                       || header_rec.eh1t1ee_eng[k] > 8000000.0)
            {
              header_rec.eh1t1ee_eng[k] = 0.0;
              header_rec.eh1t1ee_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the EE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh1t1ee_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t1ee_eng[k]);
            else printf("%9.1f ",header_rec.eh1t1ee_eng[k]);

            /******************************************************************/
            /* Print the EE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh1t1ee_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t1ee_ede[k]);
            else printf("%9.1f ",header_rec.eh1t1ee_ede[k]);

            /******************************************************************/
            /* Print the measured EE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh1t1ee_dnf[k]);

            /******************************************************************/
            /* Print the EE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh1t1ee_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the EE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh1t1ee_dnf[k] < 0.0) data_rec.eh1t1ee_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh1t1ee_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh1t1ee_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the DE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0) 
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh1t1de_eng[k] < 500.0 
                                       || header_rec.eh1t1de_eng[k] > 8000000.0)
            {
              header_rec.eh1t1de_eng[k] = 0.0;
              header_rec.eh1t1de_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the DE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh1t1de_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t1de_eng[k]);
            else printf("%9.1f ",header_rec.eh1t1de_eng[k]);

            /******************************************************************/
            /* Print the DE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh1t1de_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t1de_ede[k]);
            else printf("%9.1f ",header_rec.eh1t1de_ede[k]);

            /******************************************************************/
            /* Print the measured DE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh1t1de_dnf[k]);

            /******************************************************************/
            /* Print the DE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh1t1de_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the DE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh1t1de_dnf[k] < 0.0) data_rec.eh1t1de_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh1t1de_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh1t1de_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for HEPS-1 Telescope-1.                   */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 16 elements for the HEPS-1, Telescope-1 sample.        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The second considered is for HEPS-1, Telescope-2.                    */
      /************************************************************************/

      if (j == 2)
      {
 
        /**********************************************************************/
        /* There are 16 data values for this telescope, get ready to print    */
        /* them if needed.                                                    */
        /**********************************************************************/

        for (k=0; k<16; ++k)
        {

          /********************************************************************/
          /* Check the quality for the EE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j+1]) != 0)
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh1t2ee_eng[k] < 500.0 
                                       || header_rec.eh1t2ee_eng[k] > 8000000.0)
            {
              header_rec.eh1t2ee_eng[k] = 0.0;
              header_rec.eh1t2ee_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the EE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh1t2ee_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t2ee_eng[k]);
            else printf("%9.1f ",header_rec.eh1t2ee_eng[k]);

            /******************************************************************/
            /* Print the EE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh1t2ee_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t2ee_ede[k]);
            else printf("%9.1f ",header_rec.eh1t2ee_ede[k]);

            /******************************************************************/
            /* Print the measured EE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh1t2ee_dnf[k]);

            /******************************************************************/
            /* Print the EE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh1t2ee_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the EE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh1t2ee_dnf[k] < 0.0) data_rec.eh1t2ee_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh1t2ee_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh1t2ee_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the DE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0) 
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh1t2de_eng[k] < 500.0 
                                       || header_rec.eh1t2de_eng[k] > 8000000.0)
            {
              header_rec.eh1t2de_eng[k] = 0.0;
              header_rec.eh1t2de_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the DE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh1t2de_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t2de_eng[k]);
            else printf("%9.1f ",header_rec.eh1t2de_eng[k]);

            /******************************************************************/
            /* Print the DE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh1t2de_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh1t2de_ede[k]);
            else printf("%9.1f ",header_rec.eh1t2de_ede[k]);

            /******************************************************************/
            /* Print the measured DE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh1t2de_dnf[k]);

            /******************************************************************/
            /* Print the DE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh1t2de_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the DE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh1t2de_dnf[k] < 0.0) data_rec.eh1t2de_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh1t2de_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh1t2de_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for HEPS-1 Telescope-2.                   */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 16 elements for the HEPS-1, Telescope-2 sample.        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The third considered is for HEPS-2, Telescope-1.                     */
      /************************************************************************/

      if (j == 4)
      {
 
        /**********************************************************************/
        /* There are 16 data values for this telescope, get ready to print    */
        /* them if needed.                                                    */
        /**********************************************************************/

        for (k=0; k<16; ++k)
        {

          /********************************************************************/
          /* Check the quality for the EE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j+1]) != 0)
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh2t1ee_eng[k] < 500.0 
                                       || header_rec.eh2t1ee_eng[k] > 8000000.0)
            {
              header_rec.eh2t1ee_eng[k] = 0.0;
              header_rec.eh2t1ee_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the EE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh2t1ee_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t1ee_eng[k]);
            else printf("%9.1f ",header_rec.eh2t1ee_eng[k]);

            /******************************************************************/
            /* Print the EE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh2t1ee_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t1ee_ede[k]);
            else printf("%9.1f ",header_rec.eh2t1ee_ede[k]);

            /******************************************************************/
            /* Print the measured EE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh2t1ee_dnf[k]);

            /******************************************************************/
            /* Print the EE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh2t1ee_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the EE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh2t1ee_dnf[k] < 0.0) data_rec.eh2t1ee_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh2t1ee_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh2t1ee_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the DE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0) 
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh2t1de_eng[k] < 500.0 
                                       || header_rec.eh2t1de_eng[k] > 8000000.0)
            {
              header_rec.eh2t1de_eng[k] = 0.0;
              header_rec.eh2t1de_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the DE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh2t1de_eng[k] < 0.0) 
                                    printf("%9.1e ",header_rec.eh2t1de_eng[k]);
            else printf("%9.1f ",header_rec.eh2t1de_eng[k]);

            /******************************************************************/
            /* Print the DE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh2t1de_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t1de_ede[k]);
            else printf("%9.1f ",header_rec.eh2t1de_ede[k]);

            /******************************************************************/
            /* Print the measured DE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh2t1de_dnf[k]);

            /******************************************************************/
            /* Print the DE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh2t1de_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the DE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh2t1de_dnf[k] < 0.0) data_rec.eh2t1de_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh2t1de_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh2t1de_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for HEPS-2 Telescope-1.                   */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 16 elements for the HEPS-2, Telescope-1 sample.        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* The fourth considered is for HEPS-2, Telescope-1.                    */
      /************************************************************************/

      if (j == 6)
      {
 
        /**********************************************************************/
        /* There are 16 data values for this telescope, get ready to print    */
        /* them if needed.                                                    */
        /**********************************************************************/

        for (k=0; k<16; ++k)
        {

          /********************************************************************/
          /* Check the quality for the EE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j+1]) != 0)
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh2t2ee_eng[k] < 500.0 
                                       || header_rec.eh2t2ee_eng[k] > 8000000.0)
            {
              header_rec.eh2t2ee_eng[k] = 0.0;
              header_rec.eh2t2ee_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the EE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh2t2ee_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t2ee_eng[k]);
            else printf("%9.1f ",header_rec.eh2t2ee_eng[k]);

            /******************************************************************/
            /* Print the EE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh2t2ee_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t2ee_ede[k]);
            else printf("%9.1f ",header_rec.eh2t2ee_ede[k]);

            /******************************************************************/
            /* Print the measured EE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh2t2ee_dnf[k]);

            /******************************************************************/
            /* Print the EE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh2t2ee_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the EE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh2t2ee_dnf[k] < 0.0) data_rec.eh2t2ee_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh2t2ee_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh2t2ee_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the DE sensor.  If the quality is BAD,     */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* energy level.                                                    */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0) 
          {
              printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.eh2t2de_eng[k] < 500.0 
                                       || header_rec.eh2t2de_eng[k] > 8000000.0)
            {
              header_rec.eh2t2de_eng[k] = 0.0;
              header_rec.eh2t2de_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the DE channel center energy [eV].                       */
            /******************************************************************/

            if (header_rec.eh2t2de_eng[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t2de_eng[k]);
            else printf("%9.1f ",header_rec.eh2t2de_eng[k]);

            /******************************************************************/
            /* Print the DE channel energy width [eV].                        */
            /******************************************************************/

            if (header_rec.eh2t2de_ede[k] < 0.0) 
                                     printf("%9.1e ",header_rec.eh2t2de_ede[k]);
            else printf("%9.1f ",header_rec.eh2t2de_ede[k]);

            /******************************************************************/
            /* Print the measured DE differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].                                         */
            /******************************************************************/

            printf("%12.5e",data_rec.eh2t2de_dnf[k]);

            /******************************************************************/
            /* Print the DE RAW count which is to be used as an index into    */
            /* the HEPS error look-up table printed in the header record.     */
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.eh2t2de_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the DE measurement.  This value is */
            /* the standard deviation in the measurement.  It has the same    */
            /* units as the differential number flux (or intensity)           */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            if (data_rec.eh2t2de_dnf[k] < 0.0) data_rec.eh2t2de_raw[k] = 0;
            printf("%12.5e   ", data_rec.eh2t2de_dnf[k] * 
                               header_rec.h_err[(int) data_rec.eh2t2de_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for HEPS-2 Telescope-2.                   */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all 16 elements for the HEPS-2, Telescope-2 sample.        */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Printed all of the data for that HEPS unit and Telescope.  Got to    */
      /* next set of sensors.                                                 */
      /************************************************************************/

      printf ("\n");
    }

    /**************************************************************************/
    /* Looked at the data from all four zenith electron HEPS telescopes       */
    /* stored in that data record; need another data record.                  */
    /**************************************************************************/

    printf ("\n");
  }

  /****************************************************************************/
  /* End to data in that file.                                                */
  /****************************************************************************/

  close(file_ptr);
  printf("\nEOF\n");
}
