/******************************************************************************/
/*                                                                            */
/*  Purpose: This program reads the MEPS binary files and prints them.        */
/*           MEPS data files contain measurements retrieved from the MEPS     */
/*           sensors (both from the ion and electron channels) and cycle      */
/*           every 2.048 sec. See Winningham et al., J. Geophys. Res.,        */
/*           10649-10666, 98, 1993 for a description of the instrument.       */
/*                                                                            */
/*  Version: 1.01                                                             */
/*  Date: June 30, 1998                                                       */
/*  Code Written By: Rudy Frahm                                               */
/*                   Southwest Research Institute                             */
/*                   PO Drawer 28510                                          */
/*                   San Antonio, TX  78228                                   */
/*                   Office: (210) 522-3855                                   */
/*                   FAX: (210) 647-4325                                      */
/*                   E-mail: rudy@pemrac.space.swri.edu                       */
/*                                                                            */
/*  Purpose: This program reads the MEPS binary files and prints them.  MEPS  */
/*           data contains measurements from all of the sensors (zenith and   */
/*           nadir) for both species, electron and proton.  MEPS sensors      */
/*           cycle every 2.048 sec.  See Winningham et al., J. Geophys. Res., */
/*           10649-10666, 98, 1993 for a description of the instrument.       */
/*                                                                            */
/*  MEPS binary files have two types of records.  The first is a header       */
/*  record which occurs only at the beginning of the file.  After this header */
/*  record comes a variable number of data records.                           */
/*                                                                            */
/*  Through out the MEPS binary files, values of -1.0e+31 is used as a fill   */
/*  value.  If this value occurs in a field, it means that that field is      */
/*  invalid.  Data which has the value of 1.0e+31 has been excluded due to    */
/*  processing issues.  Numbers with exponents greater than e+31 are also     */
/*  fill values and should be excluded.                                       */
/*                                                                            */
/*  Quality values should be zero.  A value of quality which is non-zero      */
/*  means that the entire sensor data is not valid, no matter what values     */
/*  occur within the data fields.  The quality values have different values   */
/*  to indicate where failure occurred during processing and are used for     */
/*  diagnostic purposes only.                                                 */
/*                                                                            */
/*  Global Definitions:                                                       */
/*                                                                            */
/*    Header Record Structure:                                                */
/*      float em0_eng[31];   31 values of electron MEPS sensor 0 center       */
/*                           energy data in [eV]                              */
/*      float em2_eng[31];   31 values of electron MEPS sensor 2 center       */
/*                           energy data in [eV]                              */
/*      float em4_eng[31];   31 values of electron MEPS sensor 4 center       */
/*                           energy data in [eV]                              */
/*      float em6_eng[31];   31 values of electron MEPS sensor 6 center       */
/*                           energy data in [eV]                              */
/*      float em8_eng[31];   31 values of electron MEPS sensor 8 center       */
/*                           energy data in [eV]                              */
/*      float em10_eng[31];  31 values of electron MEPS sensor 10 center      */
/*                           energy data in [eV]                              */
/*      float em12_eng[31];  31 values of electron MEPS sensor 12 center      */
/*                           energy data in [eV]                              */
/*      float em14_eng[31];  31 values of electron MEPS sensor 14 center      */
/*                           energy data in [eV]                              */
/*      float pm1_eng[31];   31 values of proton MEPS sensor 1 center energy  */
/*                           data in [eV]                                     */
/*      float pm3_eng[31];   31 values of proton MEPS sensor 3 center energy  */
/*                           data in [eV]                                     */
/*      float pm5_eng[31];   31 values of proton MEPS sensor 5 center energy  */
/*                           data in [eV]                                     */
/*      float pm7_eng[31];   31 values of proton MEPS sensor 7 center energy  */
/*                           data in [eV]                                     */
/*      float pm9_eng[31];   31 values of proton MEPS sensor 9 center energy  */
/*                           data in [eV]                                     */
/*      float em0_ede[31];   31 values of electron MEPS sensor 0 energy       */
/*                           channel width [eV]                               */
/*      float em2_ede[31];   31 values of electron MEPS sensor 2 energy       */
/*                           channel width [eV]                               */
/*      float em4_ede[31];   31 values of electron MEPS sensor 4 energy       */
/*                           channel width [eV]                               */
/*      float em6_ede[31];   31 values of electron MEPS sensor 6 energy       */
/*                           channel width [eV]                               */
/*      float em8_ede[31];   31 values of electron MEPS sensor 8 energy       */
/*                           channel width [eV]                               */
/*      float em10_ede[31];  31 values of electron MEPS sensor 10 energy      */
/*                           channel width [eV]                               */
/*      float em12_ede[31];  31 values of electron MEPS sensor 12 energy      */
/*                           channel width [eV]                               */
/*      float em14_ede[31];  31 values of electron MEPS sensor 14 energy      */
/*                           channel width [eV]                               */
/*      float pm1_ede[31];   31 values of proton MEPS sensor 1 energy channel */
/*                           width [eV]                                       */
/*      float pm3_ede[31];   31 values of proton MEPS sensor 3 energy channel */
/*                           width [eV]                                       */
/*      float pm5_ede[31];   31 values of proton MEPS sensor 5 energy channel */
/*                           width [eV]                                       */
/*      float pm7_ede[31];   31 values of proton MEPS sensor 7 energy channel */
/*                           width [eV]                                       */
/*      float pm9_ede[31];   31 values of proton MEPS sensor 9 energy channel */
/*                           width [eV]                                       */
/*      float m_err[256];    256 values consisting of a fraction of error     */
/*                           assigned to each telemetry value, including      */
/*                           Poisson statistics, telemetry compression, and   */
/*                           instrument errors [unitless]                     */
/*                                                                            */
/*    Data Record Structure:                                                  */
/*      long syear;          year the data started accumulating               */
/*      long sday;           day of year the data started accumulating        */
/*      long smsec;          millisecond of day the data started accumulating */
/*      float fglat;         first latitude at center time [degrees]          */
/*      float fglon;         first longitude at center time [degrees]         */
/*      float fgalt;         first altitude at center time [km]               */
/*      float fil600;        first invariant latitude determined at center    */
/*                           time at 600 km [degrees]                         */
/*      float fmst600;       first magnetic solar time determined at center   */
/*                           time at 600 km [degrees]                         */
/*      float fsza;          first solar zenith angle at center time [degrees]*/
/*      float fpa[8];        8 values of pitch angle for first step at center */
/*                           time [degrees] - assignment (electron and proton */
/*                                sensors have the same look direction):      */
/*                           fpa[0] = MEPS detector 0: elec sen 0, prot sen 1 */
/*                           fpa[1] = MEPS detector 1: elec sen 2, prot sen 3 */
/*                           fpa[2] = MEPS detector 2: elec sen 4, prot sen 5 */
/*                           fpa[3] = MEPS detector 3: elec sen 6, prot sen 7 */
/*                           fpa[4] = MEPS detector 4: elec sen 8, prot sen 9 */
/*                           fpa[5] = MEPS detector 5: elec sen 10            */
/*                           fpa[6] = MEPS detector 6: elec sen 12            */
/*                           fpa[7] = MEPS detector 7: elec sen 14            */
/*      long eyear;          year the data stopped accumulating               */
/*      long eday;           day of year the data stopped accumulating        */
/*      long emsec;          millisecond of day the data stopped accumulating */
/*      float lglat;         last latitude at center time [degrees]           */
/*      float lglon;         last longitude at center time [degrees]          */
/*      float lgalt;         last altitude at center time [km]                */
/*      float lil600;        last invariant latitude determined at center     */
/*                           time at 600 km [degrees]                         */
/*      float lmst600;       last magnetic solar time determined at center    */
/*                           time at 600 km [degrees]                         */
/*      float lsza;          last solar zenith angle at center time [degrees] */
/*      float lpa[8];        8 values of pitch angle for last step at center  */
/*                           time [degrees] - assignment (electron and proton */
/*                                sensors have the same look direction):      */
/*                           lpa[0] = MEPS detector 0: elec sen 0, prot sen 1 */
/*                           lpa[1] = MEPS detector 1: elec sen 2, prot sen 3 */
/*                           lpa[2] = MEPS detector 2: elec sen 4, prot sen 5 */
/*                           lpa[3] = MEPS detector 3: elec sen 6, prot sen 7 */
/*                           lpa[4] = MEPS detector 4: elec sen 8, prot sen 9 */
/*                           lpa[5] = MEPS detector 5: elec sen 10            */
/*                           lpa[6] = MEPS detector 6: elec sen 12            */
/*                           lpa[7] = MEPS detector 7: elec sen 14            */
/*      float em0_dnf[31];   31 values of electron MEPS sensor 0 differential */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float em2_dnf[31];   31 values of electron MEPS sensor 2 differential */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float em4_dnf[31];   31 values of electron MEPS sensor 4 differential */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float em6_dnf[31];   31 values of electron MEPS sensor 6 differential */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float em8_dnf[31];   31 values of electron MEPS sensor 8 differential */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float em10_dnf[31];  31 values of electron MEPS sensor 10             */
/*                           differential number flux or intensity            */
/*                           [(cm**2 sr s eV)**-1]                            */
/*      float em12_dnf[31];  31 values of electron MEPS sensor 12             */
/*                           differential number flux or intensity            */
/*                           [(cm**2 sr s eV)**-1]                            */
/*      float em14_dnf[31];  31 values of electron MEPS sensor 14             */
/*                           differential number flux or intensity            */
/*                           [(cm**2 sr s eV)**-1]                            */
/*      float pm1_dnf[31];   31 values of proton MEPS sensor 1 differential   */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float pm3_dnf[31];   31 values of proton MEPS sensor 3 differential   */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float pm5_dnf[31];   31 values of proton MEPS sensor 5 differential   */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float pm7_dnf[31];   31 values of proton MEPS sensor 7 differential   */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      float pm9_dnf[31];   31 values of proton MEPS sensor 9 differential   */
/*                           number flux or intensity [(cm**2 sr s eV)**-1]   */
/*      unsigned char eq[8]; 8 values of electron sensor quality              */
/*                           assignment:                                      */
/*                           eq[0] = electron sensor 0                        */
/*                           eq[1] = electron sensor 2                        */
/*                           eq[2] = electron sensor 4                        */
/*                           eq[3] = electron sensor 6                        */
/*                           eq[4] = electron sensor 8                        */
/*                           eq[5] = electron sensor 10                       */
/*                           eq[6] = electron sensor 12                       */
/*                           eq[7] = electron sensor 14                       */
/*                           Note: values not equal to zero indicate bad      */
/*                                 and the data from that sensor should not   */
/*                                 be used                                    */
/*      unsigned char pq[5]; 5 values of proton sensor quality                */
/*                           assignment:                                      */
/*                           pq[0] = proton sensor 1                          */
/*                           pq[1] = proton sensor 3                          */
/*                           pq[2] = proton sensor 5                          */
/*                           pq[3] = proton sensor 7                          */
/*                           pq[4] = proton sensor 9                          */
/*                           Note: values not equal to zero indicate bad      */
/*                                 and the data from that sensor should not   */
/*                                 be used                                    */
/*      unsigned char em0_raw[31];                                            */
/*                           31 values of electron MEPS sensor 0 raw          */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em2_raw[31];                                            */
/*                           31 values of electron MEPS sensor 2 raw          */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em4_raw[31];                                            */
/*                           31 values of electron MEPS sensor 4 raw          */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em6_raw[31];                                            */
/*                           31 values of electron MEPS sensor 6 raw          */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em8_raw[31];                                            */
/*                           31 values of electron MEPS sensor 8 raw          */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em10_raw[31];                                           */
/*                           31 values of electron MEPS sensor 10 raw         */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em12_raw[31];                                           */
/*                           31 values of electron MEPS sensor 12 raw         */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char em14_raw[31];                                           */
/*                           31 values of electron MEPS sensor 14 raw         */
/*                           telemetry value (used in conjunction with the    */
/*                           error table stored in the header record to       */
/*                           determine the fractional error in the data       */
/*                           value)                                           */
/*      unsigned char pm1_raw[31];                                            */
/*                           31 values of proton MEPS sensor 1 raw telemetry  */
/*                           value (used in conjunction with the error table  */
/*                           stored in the header record to determine the     */
/*                           fractional error in the data value)              */
/*      unsigned char pm3_raw[31];                                            */
/*                           31 values of proton MEPS sensor 3 raw telemetry  */
/*                           value (used in conjunction with the error table  */
/*                           stored in the header record to determine the     */
/*                           fractional error in the data value)              */
/*      unsigned char pm5_raw[31];                                            */
/*                           31 values of proton MEPS sensor 5 raw telemetry  */
/*                           value (used in conjunction with the error table  */
/*                           stored in the header record to determine the     */
/*                           fractional error in the data value)              */
/*      unsigned char pm7_raw[31];                                            */
/*                           31 values of proton MEPS sensor 7 raw telemetry  */
/*                           value (used in conjunction with the error table  */
/*                           stored in the header record to determine the     */
/*                           fractional error in the data value)              */
/*      unsigned char pm9_raw[31];                                            */
/*                           31 values of proton MEPS sensor 9 raw telemetry  */
/*                           value (used in conjunction with the error table  */
/*                           stored in the header record to determine the     */
/*                           fractional error in the data value)              */
/*                                                                            */
/*  Global Variable Definitions:                                              */
/*                                                                            */
/*    int file_ptr;          pointer to the binary file (<0 means failure)    */
/*    int empty;             number of binary bites actually red (<0 means    */
/*                           failure)                                         */
/*                                                                            */
/*  Local Definitions:                                                        */
/*                                                                            */
/*    int argc;              number of command line arguments                 */
/*    char **argv;           pointer to a character string containing the     */
/*                           command line arguments                           */
/*    short btime_yr;        year of data file                                */
/*    short btime_day;       day of year of data file                         */
/*    short hr;              value of hour of day                             */
/*    short mn;              value of minute of hour                          */
/*    short sc;              value of second of minute                        */
/*    long spd;              number of seconds per day                        */
/*                           command line arguments                           */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

/* 4567 10 234567 20 234567 30 234567 40 234567 50 234567 60 234567 70 234567 */
/******************************************************************************/
/* Header Record stores the Center Energies, Energy Widths, and the error     */
/* fraction table used to determine the fraction of the differential number   */
/* which is the standard deviation.                                           */
/******************************************************************************/

struct HEAD {
  float em0_eng[31];
  float em2_eng[31];
  float em4_eng[31];
  float em6_eng[31];
  float em8_eng[31];
  float em10_eng[31];
  float em12_eng[31];
  float em14_eng[31];
  float pm1_eng[31];
  float pm3_eng[31];
  float pm5_eng[31];
  float pm7_eng[31];
  float pm9_eng[31];
  float em0_ede[31];
  float em2_ede[31];
  float em4_ede[31];
  float em6_ede[31];
  float em8_ede[31];
  float em10_ede[31];
  float em12_ede[31];
  float em14_ede[31];
  float pm1_ede[31];
  float pm3_ede[31];
  float pm5_ede[31];
  float pm7_ede[31];
  float pm9_ede[31];
  float m_err[256];
};

/******************************************************************************/
/* Data Record stores the Sweep Starting parameters (time, the O/A Values at  */
/* the start time, and the pitch angle at the start time), the Sweep Stopping */
/* parameters (time, the O/A Values at the start time, and the pitch angle at */
/* the start time), differential number flux (or intensity) Value, Sensor     */
/* quality indicator, and the RAW count (used as an index for determining the */
/* error in the measurement).                                                 */
/******************************************************************************/

struct DATA {
  long syear;
  long sday;
  long smsec;
  float fglat;
  float fglon;
  float fgalt;
  float fil600;
  float fmst600;
  float fsza;
  float fpa[8];
  long eyear;
  long eday;
  long emsec;
  float lglat;
  float lglon;
  float lgalt;
  float lil600;
  float lmst600;
  float lsza;
  float lpa[8];
  float em0_dnf[31];
  float em2_dnf[31];
  float em4_dnf[31];
  float em6_dnf[31];
  float em8_dnf[31];
  float em10_dnf[31];
  float em12_dnf[31];
  float em14_dnf[31];
  float pm1_dnf[31];
  float pm3_dnf[31];
  float pm5_dnf[31];
  float pm7_dnf[31];
  float pm9_dnf[31];
  unsigned char eq[8];
  unsigned char pq[5];
  unsigned char em0_raw[31];
  unsigned char em2_raw[31];
  unsigned char em4_raw[31];
  unsigned char em6_raw[31];
  unsigned char em8_raw[31];
  unsigned char em10_raw[31];
  unsigned char em12_raw[31];
  unsigned char em14_raw[31];
  unsigned char pm1_raw[31];
  unsigned char pm3_raw[31];
  unsigned char pm5_raw[31];
  unsigned char pm7_raw[31];
  unsigned char pm9_raw[31];
};

/******************************************************************************/
/*  Global Variable Definitions are binary file indicator variables only.     */
/******************************************************************************/

int file_ptr, empty;

void main(int argc, char **argv)
{
   long spd;
   int j,k;
   short btime_yr, btime_day;
   short hr, mn, sc;
   char bfilename[80], buf[80];
   struct HEAD header_rec;
   struct DATA data_rec;

  /****************************************************************************/
  /* Check for the proper number of command line arguments, output message if */
  /* not correct and quit the program.  If you have the correct number of     */
  /* inputs, the first is the year and the second is the day...get them.      */
  /****************************************************************************/

   if (argc != 3)
    {
       printf ("Calling sequence is: read_meps_binary YEAR DAY\n");
       printf ("...try again\n");
       exit (-1);
    }
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_yr);
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_day);

  /****************************************************************************/
  /* Formulate the correct file name and then open that file.  Output error   */
  /* message and terminate if there is a problem.                             */
  /****************************************************************************/

  /* open data files, write header here */
  sprintf(bfilename,"PEM_MEPS_%04d%03d_V01.DAT",btime_yr,btime_day);
  file_ptr = open(bfilename, O_RDONLY,0);
  if (file_ptr == -1)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not open file for output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    exit(-1);
  }

  /****************************************************************************/
  /* Read the header record.  Output error message and terminate if there is  */
  /* a problem.                                                               */
  /****************************************************************************/

  if ((empty=read(file_ptr,(char*)&header_rec,sizeof(header_rec))) < 0)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not read header record in output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    close(file_ptr);
    exit(-2);
  }

  /****************************************************************************/
  /* Write out Definitions of Column Headings.                                */
  /****************************************************************************/

  printf("Program Definitions and Units -\n");
  printf("     E = Center Energy of Channel [eV]\n");
  printf("    dE = Energy Channel Width [eV]\n");
  printf("   DNF = Differential Number Flux (or Intensity) ");
  printf("[(cm**2-s-sr-eV)**-1]\n");
  printf("   RAW = Raw Compressed Count Rate ");
  printf("in Telemetry (index for error) [unitless]\n");
  printf("  dDNF = Standard Deviation of DNF Measurement ");
  printf("[(cm**2-s-sr-eV)**-1]\n");
  printf("\n");
  printf("Notes -\n");
  printf("     1)  -1.0e+31 is used to indicate filled value\n");
  printf("     2)   1.0e+31 is used to indicate excluded value\n");
  printf("     3)  negative fluxes result from subtraction of dark current\n");
  printf("         and/or subtraction of expected contamination from protons\n");
  printf("     4)  non-zero quality values are for diagnostic purposes only ");
  printf("and indicate processing failure point\n");
  printf("     5)  zero energy values indicate multiplexed value which is not ");
  printf("available\n");
  printf("     6)  numbers with exponents greater than e+31 are filled values\n");
  printf("\n");

  /****************************************************************************/
  /* Loop through reading the data records and read them.  terminate the      */
  /* program when the all have been read.                                     */
  /****************************************************************************/

  while ((empty=read(file_ptr,(char*)&data_rec,sizeof(data_rec))) > 0)
  {

    /**************************************************************************/
    /* For the Starting sweep time, convert the millisec per day into hours,  */
    /* minutes and seconds.  Print all starting sweep time values.            */
    /**************************************************************************/

    spd = data_rec.smsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Starting at - %04ld%03ld/%02d:%02d:%02d(%ld msec): ",
            data_rec.syear,data_rec.sday,hr,mn,sc,data_rec.smsec);
    printf("First Sweep Step Parameters (center time) ->\n");
    if (data_rec.fgalt < 0.0 || data_rec.fgalt > 1000.0)
                              printf("     Spacecraft Altitude = BAD,        ");
    else printf("     Spacecraft Altitude = %6.2f km,  ",data_rec.fgalt);
    if (data_rec.fsza < 0.0 || data_rec.fsza > 360.0)
                                           printf("Solar Zenith Angle = BAD\n");
    else printf("Solar Zenith Angle = %7.2f deg\n",data_rec.fsza);
    if (data_rec.fglat < -90.0 || data_rec.fglat > 90.0)
                               printf("     Geodetic Latitude = BAD,         ");
    else printf("     Geodetic Latitude = %6.2f deg,  ",data_rec.fglat);
    if (data_rec.fglon < 0.0 || data_rec.fglon > 360.0)
                                               printf("East Longitude = BAD\n");
    else printf("East Longitude = %7.2f deg\n",data_rec.fglon);
    if (data_rec.fil600 < -90.0 || data_rec.fil600 > 90.0)
                   printf("     At 600 km: Invariant Latitude = BAD,         ");
    else printf("     At 600 km: Invariant Latitude = %6.2f deg,  ",
                                                               data_rec.fil600);
    if (data_rec.fmst600 < 0.0 || data_rec.fmst600 > 24.0)
                                          printf("Magnetic Solar Time = BAD\n");
    else printf("Magnetic Solar Time = %6.2f hr\n",data_rec.fmst600);
    printf ("\n");

    /**************************************************************************/
    /* For the Ending sweep time, convert the millisec per day into hours,    */
    /* minutes and seconds.  Print all ending sweep time values.              */
    /**************************************************************************/

    spd = data_rec.emsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    printf("Ending at - %04ld%03ld/%02d:%02d:%02d(%ld msec): ",
            data_rec.eyear,data_rec.eday,hr,mn,sc,data_rec.emsec);
    printf("Last Sweep Step Parameters (center time) ->\n");
    if (data_rec.lgalt < 0.0 || data_rec.lgalt > 1000.0)
                              printf("     Spacecraft Altitude = BAD,        ");
    else printf("     Spacecraft Altitude = %6.2f km,  ",data_rec.lgalt);
    if (data_rec.lsza < 0.0 || data_rec.lsza > 360.0)
                                           printf("Solar Zenith Angle = BAD\n");
    else printf("Solar Zenith Angle = %7.2f deg\n",data_rec.lsza);
    if (data_rec.lglat < -90.0 || data_rec.lglat > 90.0)
                               printf("     Geodetic Latitude = BAD,         ");
    else printf("     Geodetic Latitude = %6.2f deg,  ",data_rec.lglat);
    if (data_rec.lglon < 0.0 || data_rec.lglon > 360.0)
                                               printf("East Longitude = BAD\n");
    else printf("East Longitude = %7.2f deg\n",data_rec.lglon);
    if (data_rec.lil600 < -90.0 || data_rec.lil600 > 90.0)
                   printf("     At 600 km: Invariant Latitude = BAD,         ");
    else printf("     At 600 km: Invariant Latitude = %6.2f deg,  ",
                                                               data_rec.lil600);
    if (data_rec.lmst600 < 0.0 || data_rec.lmst600 > 24.0)
                                          printf("Magnetic Solar Time = BAD\n");
    else printf("Magnetic Solar Time = %6.2f hr\n",data_rec.lmst600);
    printf ("\n");

    /**************************************************************************/
    /* Go through each sensors' data a telescope at a time.  Decipher and     */
    /* print the result.                                                      */
    /**************************************************************************/

    for (j=0; j<8; ++j)
    {

      /************************************************************************/
      /* Print the MEPS unit.  Print the pitch angle at the first and last    */
      /* last sweep step.  The range of the pitch angle is from 0 to 180 deg. */
      /* Any number out side of this range means the quality is not valid and */
      /* the word "BAD" is printed.                                           */
      /************************************************************************/

      printf("MEPS Detector %02d:  E (eV)  dE (eV)  ",j);
      printf("DNF (cm**2-s-sr-eV)**-1  (RAW COUNT)  dDNF (cm**2-s-sr-eV)**-1\n");
      printf("Pitch Angle: ");
      if (data_rec.fpa[j] >= 0.0 && data_rec.fpa[j] <= 180.0) 
                printf(" %8.3f deg First Sweep ",data_rec.fpa[j]);
      else printf("BAD First Sweep ");
      printf("<--> ");
      if (data_rec.lpa[j] >= 0.0 && data_rec.lpa[j] <= 180.0) 
                printf(" %8.3f deg Last Sweep ",data_rec.lpa[j]);
      else printf("BAD Last Sweep ");
      printf ("\n");

      /************************************************************************/
      /* The quality values of the MEPS electron and (if available) proton    */
      /* sensor(s) are evaluated.  If they are not 0, a error during          */
      /* processing was indicated.  If the quality in non-zero, the word      */
      /* "BAD" is printed.                                                    */
      /************************************************************************/

      printf("MEPS Electron Sensor %02d QUALITY ",2*j);
      if (((int) data_rec.eq[j]) != 0) printf("BAD  ");
      else printf("%03d  ",(int) data_rec.eq[j]);
      if (j < 5) 
      {
        printf("          ");
        printf("   MEPS Proton Sensor %02d QUALITY ",2*j+1);
        if (((int) data_rec.pq[j]) != 0) printf("BAD  ");
        else printf("%03d  ",(int) data_rec.pq[j]);
      }
      printf ("\n");

      /************************************************************************/
      /* If the quality of both the active MEPS electron and proton sensors   */
      /* are bad, don't print out any values, just get the next sensor's data */
      /* but don't forget that the nadir sensors have no proton component.    */
      /************************************************************************/

      if (j < 5)
      {
        if ((((int) data_rec.eq[j]) != 0) && (((int) data_rec.pq[j]) != 0)) 
        {
          printf ("\n");
          continue;
        }
      }
      else 
      {
        if (((int) data_rec.eq[j]) != 0) 
        {
          printf ("\n");
          continue;
        }
      }

      /************************************************************************/
      /* Print headings for electron parameters if the quality of that sensor */
      /* is good.                                                             */
      /************************************************************************/

      if (((int) data_rec.eq[j]) != 0)
        printf("                                                   ");
      else
        printf("    E         dE        DNF   (RAW)    dDNF      ");

      /************************************************************************/
      /* Print headings for proton parameters if the quality of that sensor   */
      /* is good.                                                             */
      /************************************************************************/

      if (j < 5)
      {
        if (((int) data_rec.pq[j]) != 0)
          printf("                                                   ");
        else
          printf("    E         dE        DNF   (RAW)    dDNF     ");
      }
      printf ("\n");

      /************************************************************************/
      /* Specific values are printed depending on which MEPS unit being       */
      /* examined.  They have to be handled separately.  The first considered */
      /* is for MEPS head 1, electrons and protons.                           */
      /************************************************************************/

      if (j == 0)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron sensor, but only 15 are telemetered   */
        /* for the proton sensor.  This sensor is multiplexed and its         */
        /* availability is controlled by the orientation of the spacecraft.   */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* Check the quality for the electron sensor.  If the quality is    */
          /* BAD, a blank line is needed; otherwise values may be printed at  */
          /* this level.                                                      */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.em0_eng[k] < 0.1 || header_rec.em0_eng[k] > 40000.0)
            {
              header_rec.em0_eng[k] = 0.0;
              header_rec.em0_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the electron channel center energy [eV].                 */
            /******************************************************************/

            if (header_rec.em0_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em0_eng[k]);
            else printf("%9.2f ",header_rec.em0_eng[k]);

            /******************************************************************/
            /* Print the electron channel energy width [eV].                  */
            /******************************************************************/

            if (header_rec.em0_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em0_ede[k]);
            else printf("%9.2f ",header_rec.em0_ede[k]);

            /******************************************************************/
            /* Print the measured electron differential number flux (or       */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.em0_dnf[k]);

            /******************************************************************/
            /* Print the electron RAW count which is to be used as an index   */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.em0_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the electron measurement.  This    */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.em0_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.em0_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the proton sensor.  If the quality is good */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* level.                                                           */
          /********************************************************************/

          if (((int) data_rec.pq[j]) == 0)
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.pm1_eng[k] < 0.1 || header_rec.pm1_eng[k] > 40000.0)
            {
              header_rec.pm1_eng[k] = 0.0;
              header_rec.pm1_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the proton channel center energy [eV].                   */
            /******************************************************************/

            if (header_rec.pm1_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm1_eng[k]);
            else printf("%9.2f ",header_rec.pm1_eng[k]);

            /******************************************************************/
            /* Print the proton channel energy width [eV].                    */
            /******************************************************************/

            if (header_rec.pm1_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm1_ede[k]);
            else printf("%9.2f ",header_rec.pm1_ede[k]);

            /******************************************************************/
            /* Print the measured proton differential number flux (or         */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.pm1_dnf[k]);

            /******************************************************************/
            /* Print the proton RAW count which is to be used as an index     */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.pm1_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the proton measurement.  This      */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.pm1_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.pm1_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for MEPS unit 1                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 1 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 2, electrons and protons.  */
      /************************************************************************/

      else if (j == 1)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron sensor, but only 15 are telemetered   */
        /* for the proton sensor.  This sensor is multiplexed and its         */
        /* availability is controlled by the orientation of the spacecraft.   */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* Check the quality for the electron sensor.  If the quality is    */
          /* BAD, a blank line is needed; otherwise values may be printed at  */
          /* this level.                                                      */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.em2_eng[k] < 0.1 || header_rec.em2_eng[k] > 40000.0)
            {
              header_rec.em2_eng[k] = 0.0;
              header_rec.em2_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the electron channel center energy [eV].                 */
            /******************************************************************/

            if (header_rec.em2_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em2_eng[k]);
            else printf("%9.2f ",header_rec.em2_eng[k]);

            /******************************************************************/
            /* Print the electron channel energy width [eV].                  */
            /******************************************************************/

            if (header_rec.em2_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em2_ede[k]);
            else printf("%9.2f ",header_rec.em2_ede[k]);

            /******************************************************************/
            /* Print the measured electron differential number flux (or       */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.em2_dnf[k]);

            /******************************************************************/
            /* Print the electron RAW count which is to be used as an index   */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.em2_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the electron measurement.  This    */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.em2_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.em2_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the proton sensor.  If the quality is good */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* level.                                                           */
          /********************************************************************/

          if (((int) data_rec.pq[j]) == 0)
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.pm3_eng[k] < 0.1 || header_rec.pm3_eng[k] > 40000.0)
            {
              header_rec.pm3_eng[k] = 0.0;
              header_rec.pm3_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the proton channel center energy [eV].                   */
            /******************************************************************/

            if (header_rec.pm3_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm3_eng[k]);
            else printf("%9.2f ",header_rec.pm3_eng[k]);

            /******************************************************************/
            /* Print the proton channel energy width [eV].                    */
            /******************************************************************/

            if (header_rec.pm3_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm3_ede[k]);
            else printf("%9.2f ",header_rec.pm3_ede[k]);

            /******************************************************************/
            /* Print the measured proton differential number flux (or         */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.pm3_dnf[k]);

            /******************************************************************/
            /* Print the proton RAW count which is to be used as an index     */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.pm3_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the proton measurement.  This      */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.pm3_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.pm3_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for MEPS head 2                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 2 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 3, electrons and protons.  */
      /************************************************************************/

      else if (j == 2)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for both the electron and proton sensor, but this      */
        /* sensor is multiplexed and its availability is controlled by the    */
        /* orientation of the spacecraft.                                     */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* Check the quality for the electron sensor.  If the quality is    */
          /* BAD, a blank line is needed; otherwise values may be printed at  */
          /* this level.                                                      */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.em4_eng[k] < 0.1 || header_rec.em4_eng[k] > 40000.0)
            {
              header_rec.em4_eng[k] = 0.0;
              header_rec.em4_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the electron channel center energy [eV].                 */
            /******************************************************************/

            if (header_rec.em4_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em4_eng[k]);
            else printf("%9.2f ",header_rec.em4_eng[k]);

            /******************************************************************/
            /* Print the electron channel energy width [eV].                  */
            /******************************************************************/

            if (header_rec.em4_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em4_ede[k]);
            else printf("%9.2f ",header_rec.em4_ede[k]);

            /******************************************************************/
            /* Print the measured electron differential number flux (or       */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.em4_dnf[k]);

            /******************************************************************/
            /* Print the electron RAW count which is to be used as an index   */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.em4_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the electron measurement.  This    */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.em4_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.em4_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the proton sensor.  If the quality is good */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* level.                                                           */
          /********************************************************************/

          if (((int) data_rec.pq[j]) == 0)
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.pm5_eng[k] < 0.1 || header_rec.pm5_eng[k] > 40000.0)
            {
              header_rec.pm5_eng[k] = 0.0;
              header_rec.pm5_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the proton channel center energy [eV].                   */
            /******************************************************************/

            if (header_rec.pm5_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm5_eng[k]);
            else printf("%9.2f ",header_rec.pm5_eng[k]);

            /******************************************************************/
            /* Print the proton channel energy width [eV].                    */
            /******************************************************************/

            if (header_rec.pm5_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm5_ede[k]);
            else printf("%9.2f ",header_rec.pm5_ede[k]);

            /******************************************************************/
            /* Print the measured proton differential number flux (or         */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.pm5_dnf[k]);

            /******************************************************************/
            /* Print the proton RAW count which is to be used as an index     */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.pm5_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the proton measurement.  This      */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.pm5_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.pm5_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for MEPS head 3                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 3 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 4, electrons and protons.  */
      /************************************************************************/

      else if (j == 3)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron sensor, but only 15 are telemetered   */
        /* for the proton sensor.  These sensors are always available.        */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* Check the quality for the electron sensor.  If the quality is    */
          /* BAD, a blank line is needed; otherwise values may be printed at  */
          /* this level.                                                      */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.em6_eng[k] < 0.1 || header_rec.em6_eng[k] > 40000.0)
            {
              header_rec.em6_eng[k] = 0.0;
              header_rec.em6_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the electron channel center energy [eV].                 */
            /******************************************************************/

            if (header_rec.em6_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em6_eng[k]);
            else printf("%9.2f ",header_rec.em6_eng[k]);

            /******************************************************************/
            /* Print the electron channel energy width [eV].                  */
            /******************************************************************/

            if (header_rec.em4_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em6_ede[k]);
            else printf("%9.2f ",header_rec.em6_ede[k]);

            /******************************************************************/
            /* Print the measured electron differential number flux (or       */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.em6_dnf[k]);

            /******************************************************************/
            /* Print the electron RAW count which is to be used as an index   */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.em6_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the electron measurement.  This    */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.em6_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.em6_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the proton sensor.  If the quality is good */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* level.                                                           */
          /********************************************************************/

          if (((int) data_rec.pq[j]) == 0)
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.pm7_eng[k] < 0.1 || header_rec.pm7_eng[k] > 40000.0)
            {
              header_rec.pm7_eng[k] = 0.0;
              header_rec.pm7_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the proton channel center energy [eV].                   */
            /******************************************************************/

            if (header_rec.pm7_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm7_eng[k]);
            else printf("%9.2f ",header_rec.pm7_eng[k]);

            /******************************************************************/
            /* Print the proton channel energy width [eV].                    */
            /******************************************************************/

            if (header_rec.pm7_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm7_ede[k]);
            else printf("%9.2f ",header_rec.pm7_ede[k]);

            /******************************************************************/
            /* Print the measured proton differential number flux (or         */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.pm7_dnf[k]);

            /******************************************************************/
            /* Print the proton RAW count which is to be used as an index     */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.pm7_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the proton measurement.  This      */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.pm7_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.pm7_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for MEPS head 4                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 4 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 5, electrons and protons.  */
      /************************************************************************/

      else if (j == 4)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron sensor, but only 15 are telemetered   */
        /* for the proton sensor.  These sensors are always available.        */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* Check the quality for the electron sensor.  If the quality is    */
          /* BAD, a blank line is needed; otherwise values may be printed at  */
          /* this level.                                                      */
          /********************************************************************/

          if (((int) data_rec.eq[j]) != 0)
          {
            printf("                                                   ");
          }
          else
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.em8_eng[k] < 0.1 || header_rec.em8_eng[k] > 40000.0)
            {
              header_rec.em8_eng[k] = 0.0;
              header_rec.em8_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the electron channel center energy [eV].                 */
            /******************************************************************/

            if (header_rec.em8_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em8_eng[k]);
            else printf("%9.2f ",header_rec.em8_eng[k]);

            /******************************************************************/
            /* Print the electron channel energy width [eV].                  */
            /******************************************************************/

            if (header_rec.em8_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.em8_ede[k]);
            else printf("%9.2f ",header_rec.em8_ede[k]);

            /******************************************************************/
            /* Print the measured electron differential number flux (or       */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.em8_dnf[k]);

            /******************************************************************/
            /* Print the electron RAW count which is to be used as an index   */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.em8_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the electron measurement.  This    */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.em8_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.em8_raw[k]]);
          }

          /********************************************************************/
          /* Check the quality for the proton sensor.  If the quality is good */
          /* a blank line is needed; otherwise values may be printed at this  */
          /* level.                                                           */
          /********************************************************************/

          if (((int) data_rec.pq[j]) == 0)
          {

            /******************************************************************/
            /* If energies are out of range, that energy channel is not       */
            /* valid.  Indicate this by setting the channel energy and width  */
            /* to zero.                                                       */
            /******************************************************************/

            if (header_rec.pm9_eng[k] < 0.1 || header_rec.pm9_eng[k] > 40000.0)
            {
              header_rec.pm9_eng[k] = 0.0;
              header_rec.pm9_ede[k] = 0.0;
            }

            /******************************************************************/
            /* Print the proton channel center energy [eV].                   */
            /******************************************************************/

            if (header_rec.pm9_eng[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm9_eng[k]);
            else printf("%9.2f ",header_rec.pm9_eng[k]);

            /******************************************************************/
            /* Print the proton channel energy width [eV].                    */
            /******************************************************************/

            if (header_rec.pm9_ede[k] < 0.0) 
                                         printf("%9.1e ",header_rec.pm9_ede[k]);
            else printf("%9.2f ",header_rec.pm9_ede[k]);

            /******************************************************************/
            /* Print the measured proton differential number flux (or         */
            /* intensity) [(cm**2-sr-s-eV)**-1].                              */
            /******************************************************************/

            printf("%10.3e",data_rec.pm9_dnf[k]);

            /******************************************************************/
            /* Print the proton RAW count which is to be used as an index     */
            /* into the MEPS error look-up table printed in the header record.*/
            /******************************************************************/

            printf("(%03d) ",(int) data_rec.pm9_raw[k]);

            /******************************************************************/
            /* Print the error estimate in the proton measurement.  This      */
            /* value is the standard deviation in the measurement.  It has    */
            /* the same units as the differential number flux (or intensity)  */
            /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.        */
            /******************************************************************/

            printf("%10.3e   ",data_rec.pm9_dnf[k] * 
                                   header_rec.m_err[(int) data_rec.pm9_raw[k]]);
          }

          /********************************************************************/
          /* Printed a line of data for MEPS head 5                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 5 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 6, electrons.              */
      /************************************************************************/

      else if (j == 5)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron.  There is no proton sensor.  Data    */
        /* from this detector is always available.                            */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* If energies are out of range, that energy channel is not valid.  */
          /* Indicate this by setting the channel energy and width to zero.   */
          /********************************************************************/

          if (header_rec.em10_eng[k] < 0.1 || header_rec.em10_eng[k] > 40000.0)
          {
            header_rec.em10_eng[k] = 0.0;
            header_rec.em10_ede[k] = 0.0;
          }

          /********************************************************************/
          /* Print the electron channel center energy [eV].                   */
          /********************************************************************/

          if (header_rec.em10_eng[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em10_eng[k]);
          else printf("%9.2f ",header_rec.em10_eng[k]);

          /********************************************************************/
          /* Print the electron channel energy width [eV].                    */
          /********************************************************************/

          if (header_rec.em10_ede[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em10_ede[k]);
          else printf("%9.2f ",header_rec.em10_ede[k]);

          /********************************************************************/
          /* Print the measured electron differential number flux (or         */
          /* intensity) [(cm**2-sr-s-eV)**-1].                                */
          /********************************************************************/

          printf("%10.3e",data_rec.em10_dnf[k]);

          /********************************************************************/
          /* Print the electron RAW count which is to be used as an index     */
          /* into the MEPS error look-up table printed in the header record.  */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.em10_raw[k]);

          /********************************************************************/
          /* Print the error estimate in the electron measurement.  This      */
          /* value is the standard deviation in the measurement.  It has      */
          /* the same units as the differential number flux (or intensity)    */
          /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.          */
          /********************************************************************/

          printf("%10.3e   ",data_rec.em10_dnf[k] * 
                                  header_rec.m_err[(int) data_rec.em10_raw[k]]);

          /********************************************************************/
          /* Printed a line of data for MEPS head 6                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 6 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 7, electrons.              */
      /************************************************************************/

      else if (j == 6)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron.  There is no proton sensor.  Data    */
        /* from this detector is always available.                            */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* If energies are out of range, that energy channel is not valid.  */
          /* Indicate this by setting the channel energy and width to zero.   */
          /********************************************************************/

          if (header_rec.em12_eng[k] < 0.1 || header_rec.em12_eng[k] > 40000.0)
          {
            header_rec.em12_eng[k] = 0.0;
            header_rec.em12_ede[k] = 0.0;
          }

          /********************************************************************/
          /* Print the electron channel center energy [eV].                   */
          /********************************************************************/

          if (header_rec.em12_eng[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em12_eng[k]);
          else printf("%9.2f ",header_rec.em12_eng[k]);

          /********************************************************************/
          /* Print the electron channel energy width [eV].                    */
          /********************************************************************/

          if (header_rec.em12_ede[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em12_ede[k]);
          else printf("%9.2f ",header_rec.em12_ede[k]);

          /********************************************************************/
          /* Print the measured electron differential number flux (or         */
          /* intensity) [(cm**2-sr-s-eV)**-1].                                */
          /********************************************************************/

          printf("%10.3e",data_rec.em12_dnf[k]);

          /********************************************************************/
          /* Print the electron RAW count which is to be used as an index     */
          /* into the MEPS error look-up table printed in the header record.  */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.em12_raw[k]);

          /********************************************************************/
          /* Print the error estimate in the electron measurement.  This      */
          /* value is the standard deviation in the measurement.  It has      */
          /* the same units as the differential number flux (or intensity)    */
          /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.          */
          /********************************************************************/

          printf("%10.3e   ",data_rec.em12_dnf[k] * 
                                  header_rec.m_err[(int) data_rec.em12_raw[k]]);

          /********************************************************************/
          /* Printed a line of data for MEPS head 7                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 7 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Specific values are printed for MEPS unit 8, electrons.              */
      /************************************************************************/

      else if (j == 7)
      {

        /**********************************************************************/
        /* There are 31 data values for the MEPS sweep.  All 31 steps are     */
        /* telemetered for the electron.  There is no proton sensor.  Data    */
        /* from this detector is multiplexed and its availability is          */
        /* controlled by the orientation of the spacecraft.                   */
        /**********************************************************************/

        for (k=0; k<31; ++k)
        {

          /********************************************************************/
          /* If energies are out of range, that energy channel is not valid.  */
          /* Indicate this by setting the channel energy and width to zero.   */
          /********************************************************************/

          if (header_rec.em14_eng[k] < 0.1 || header_rec.em14_eng[k] > 40000.0)
          {
            header_rec.em14_eng[k] = 0.0;
            header_rec.em14_ede[k] = 0.0;
          }

          /********************************************************************/
          /* Print the electron channel center energy [eV].                   */
          /********************************************************************/

          if (header_rec.em14_eng[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em14_eng[k]);
          else printf("%9.2f ",header_rec.em14_eng[k]);

          /********************************************************************/
          /* Print the electron channel energy width [eV].                    */
          /********************************************************************/

          if (header_rec.em14_ede[k] < 0.0) 
                                        printf("%9.1e ",header_rec.em14_ede[k]);
          else printf("%9.2f ",header_rec.em14_ede[k]);

          /********************************************************************/
          /* Print the measured electron differential number flux (or         */
          /* intensity) [(cm**2-sr-s-eV)**-1].                                */
          /********************************************************************/

          printf("%10.3e",data_rec.em14_dnf[k]);

          /********************************************************************/
          /* Print the electron RAW count which is to be used as an index     */
          /* into the MEPS error look-up table printed in the header record.  */
          /********************************************************************/

          printf("(%03d) ",(int) data_rec.em14_raw[k]);

          /********************************************************************/
          /* Print the error estimate in the electron measurement.  This      */
          /* value is the standard deviation in the measurement.  It has      */
          /* the same units as the differential number flux (or intensity)    */
          /* [(cm**2-sr-s-eV)**-1].  100 % error in negative values.          */
          /********************************************************************/

          printf("%10.3e   ",data_rec.em14_dnf[k] * 
                                  header_rec.m_err[(int) data_rec.em14_raw[k]]);

          /********************************************************************/
          /* Printed a line of data for MEPS head 8                           */
          /********************************************************************/

          printf ("\n");
        }

        /**********************************************************************/
        /* Printed all elements of the MEPS unit 8 sample                     */
        /**********************************************************************/

      }

      /************************************************************************/
      /* Printed all of the data for that MEPS unit.  Got to next set of      */
      /* sensors.                                                             */
      /************************************************************************/

      printf ("\n");
    }

    /**************************************************************************/
    /* Looked at the data from all MEPS sensors; need another data record.    */
    /**************************************************************************/

    printf ("\n");
  }

  /****************************************************************************/
  /* End to data in that file.                                                */
  /****************************************************************************/

  close(file_ptr);
  printf("\nEOF\n");
}
