/******************************************************************************/
/*                                                                            */
/*  Version: 1.00                                                             */
/*  Date: July 6, 2006                                                        */
/*  Code Written By: Rudy Frahm                                               */
/*                   Southwest Research Institute                             */
/*                   PO Drawer 28510                                          */
/*                   San Antonio, TX  78228                                   */
/*                   Office: (210) 522-3855                                   */
/*                   FAX: (210) 647-4325                                      */
/*                   E-mail: rfrahm@swri.edu                                  */
/*                                                                            */
/*  Purpose: This program reads the VMAGAC binary files and prints them.      */
/*           VMAGAC data contains AC measurements retrieved from the VMAG     */
/*           sensor which is mounted on the UARS zenith boom.  VMAG data is   */
/*           sampled five times per second.  See Winningham et al., J.        */
/*           Geophys. Res., 10649-10666, 98, 1993 for a description of the    */
/*           instrument.                                                      */
/*                                                                            */
/*  VMAGAC binary files have two types of records.  The first is a header     */
/*  record which occurs only at the beginning of the file.  After this header */
/*  record comes a variable number of data records.                           */
/*                                                                            */
/*  Throughout the VMAGAC binary files, values of -1.0e-31 is used as a fill  */
/*  value.  If this value occurs in a field, it means that that field is      */
/*  invalid.                                                                  */
/*                                                                            */
/*  Quality values should be zero.  A value of quality which is non-zero      */
/*  means that the entire sensor data is not valid, no matter what values     */
/*  occur within the data fields.  The quality values are different           */
/*  to indicate where failure occurred during processing and are used for     */
/*  diagnostic purposes only.                                                 */
/*                                                                            */
/*                                                                            */
/*  Global Structure Definitions:                                             */
/*                                                                            */
/*    Header Record Structure:                                                */
/*      float vmag_zboom_11;  Contains the 1, 1 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_21;  Contains the 2, 1 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_31;  Contains the 3, 1 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_12;  Contains the 1, 2 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_22;  Contains the 2, 2 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_32;  Contains the 3, 2 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_13;  Contains the 1, 3 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_23;  Contains the 2, 3 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float vmag_zboom_33;  Contains the 3, 3 rotational component relating */
/*                            the VMAG frame to the zenith boom frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_11;  Contains the 1, 1 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_21;  Contains the 2, 1 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_31;  Contains the 3, 1 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_12;  Contains the 1, 2 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_22;  Contains the 2, 2 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_32;  Contains the 3, 2 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_13;  Contains the 1, 3 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_23;  Contains the 2, 3 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*      float zboom_uars_33;  Contains the 3, 3 rotational component relating */
/*                            the zenith boom frame to the UARS frame         */
/*                            (FORTRAN specification system)                  */
/*                                                                            */
/*    Data Record Structure:                                                  */
/*      long syear;          year the data sample                             */
/*      long sday;           day of year the data sample                      */
/*      long smsec;          millisecond of day the data sample               */
/*      float cglat;         latitude at center of sweep [degrees]            */
/*      float cglon;         longitude at center of sweep [degrees]           */
/*      float cgalt;         altitude at center of sweep [km]                 */
/*      float cil600;        invariant latitude determined at center          */
/*                           of sweep for 600 km [degrees]                    */
/*      float cmst600;       magnetic solar time determined at center         */
/*                           of sweep for 600 km [degrees]                    */
/*      float csza;          solar zenith angle at center of sweep [degrees]  */
/*      float uars_eci_11;   Contains the 1, 1 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_21;   Contains the 2, 1 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_31;   Contains the 3, 1 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_12;   Contains the 1, 2 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_22;   Contains the 2, 2 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_32;   Contains the 3, 2 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_13;   Contains the 1, 3 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_23;   Contains the 2, 3 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float uars_eci_33;   Contains the 3, 3 rotational component relating  */
/*                           the UARS frame to the ECI (Earth Centered        */
/*                           Inertial) frame (FORTRAN specification system)   */
/*      float pem_vmag_temp; temperature of the VMAG sensor [degC]            */
/*      float pem_vmag_ac_nT[3];  VMAG AC vector magnetic field [nT],         */
/*                                 assignments are as follows:                */
/*                                 pem_vmag_ac_nT[0] = AC x-component (Bx)    */
/*                                 pem_vmag_ac_nT[1] = AC y-component (By)    */
/*                                 pem_vmag_ac_nT[2] = AC z-component (Bz)    */
/*      unsigned char pem_vmag_ac_raw[3]; raw component values of the VMAG AC */
/*                                        vector magnetic field (compressed)  */
/*                                        telemetry value [unitless],         */
/*                                        assignments are as follows:         */
/*                                        pem_vmag_ac_raw[0] = AC x-component */
/*                                        pem_vmag_ac_raw[1] = AC y-component */
/*                                        pem_vmag_ac_raw[2] = AC z-component */
/*      unsigned char pem_vmag_ac_q[3]; sensor quality values of the VMAG AC  */
/*                                      vector, assignments are as follows:   */
/*                                      pem_vmag_ac_q[0] = AC x-component     */
/*                                      pem_vmag_ac_q[1] = AC y-component     */
/*                                      pem_vmag_ac_q[2] = AC z-component     */
/*      unsigned char pad[2]: for alignment on 4-byte boundaries              */
/*                                                                            */
/*  Global Variable Definitions:                                              */
/*                                                                            */
/*    int file_ptr;          pointer to the binary file (<0 means failure)    */
/*    int empty;             number of binary bites actually read (<0 means   */
/*                           failure)                                         */
/*                                                                            */
/*  Local Variable Definitions:                                               */
/*                                                                            */
/*    int argc;              number of command line arguments                 */
/*    char **argv;           pointer to a character string containing the     */
/*                           command line arguments                           */
/*    short btime_yr;        year of data file                                */
/*    short btime_day;       day of year of data file                         */
/*    short hr;              value of hour of day                             */
/*    short mn;              value of minute of hour                          */
/*    short sc;              value of second of minute                        */
/*    long spd;              number of seconds per day                        */
/*    float lat;             current latitude value                           */
/*    float lon;             current longitude value                          */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

/* 4567 10 234567 20 234567 30 234567 40 234567 50 234567 60 234567 70 234567 */
/******************************************************************************/
/* Header Record stores the coordinate transformations required to map the    */
/* VMAG sensor components into the frame of the zenith boom and the UARS      */
/* spacecraft.                                                                */
/******************************************************************************/

struct HEAD {
  float vmag_zboom_11;
  float vmag_zboom_21;
  float vmag_zboom_31;
  float vmag_zboom_12;
  float vmag_zboom_22;
  float vmag_zboom_32;
  float vmag_zboom_13;
  float vmag_zboom_23;
  float vmag_zboom_33;
  float zboom_uars_11;
  float zboom_uars_21;
  float zboom_uars_31;
  float zboom_uars_12;
  float zboom_uars_22;
  float zboom_uars_32;
  float zboom_uars_13;
  float zboom_uars_23;
  float zboom_uars_33;
};

/******************************************************************************/
/* Data Record stores the Sample time, Center O/A Values, the UARS to ECI     */
/* coordinate matrix at the spacecraft position, VMAG sensor temperature, the */
/* three components of the vector VMAG AC magnetic field, the three components*/
/* of the vector VMAG AC magnetic raw telemetry values, the three components  */
/* of the vector VMAG AC magnetic quality.                                    */
/******************************************************************************/

struct DATA {
  long syear;
  long sday;
  long smsec;
  float cglat;
  float cglon;
  float cgalt;
  float cil600;
  float cmst600;
  float csza;
  float uars_eci_11;
  float uars_eci_21;
  float uars_eci_31;
  float uars_eci_12;
  float uars_eci_22;
  float uars_eci_32;
  float uars_eci_13;
  float uars_eci_23;
  float uars_eci_33;
  float pem_vmag_temp;
  float pem_vmag_ac_nT[3];
  unsigned char pem_vmag_ac_raw[3];
  unsigned char pem_vmag_ac_q[3];
  unsigned char pad[2];
};

/******************************************************************************/
/*  Global Variable Definitions are binary file indicator variables only.     */
/******************************************************************************/

int file_ptr, empty;

int main(int argc, char **argv)
{
  float lat, lon;
  long spd;
  int j,k;
  short btime_yr, btime_day;
  short hr, mn, sc, fc;
  char bfilename[80], buf[80];
  struct HEAD header_rec;
  struct DATA data_rec;

  /****************************************************************************/
  /* Check for the proper number of command line arguments, output message if */
  /* not correct and quit the program.  If you have the correct number of     */
  /* inputs, the first is the year and the second is the day...get them.      */
  /****************************************************************************/

  if (argc != 3)
   {
      printf ("Calling sequence is: read_vmagac_binary YEAR DAY\n");
      printf ("...try again\n");
      exit (-1);
   }
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_yr);
   strcpy (buf, *++argv);
   sscanf(buf,"%hd",&btime_day);

  /****************************************************************************/
  /* Formulate the correct file name and then open that file.  Output error   */
  /* message and terminate if there is a problem.                             */
  /****************************************************************************/

  sprintf(bfilename,"PEM_VMAG_AC_%04d%03d_V01.DAT",btime_yr,btime_day);
  file_ptr = open(bfilename, O_RDONLY,0);
  if (file_ptr == -1)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not open file for output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    exit(-1);
  }

  /****************************************************************************/
  /* Read the header record.  Output error message and terminate if there is  */
  /* a problem.                                                               */
  /****************************************************************************/

  if ((empty=read(file_ptr,(char*)&header_rec,sizeof(header_rec))) < 0)
  {
    printf("\n\n");
    printf("***** FATAL ERROR *****\n");
    printf("\n\n");
    printf("     Could not read header record in output, ");
    printf("file name: %s\n",bfilename);
    printf("     Process terminated prematurely\n");
    printf("\n\n");
    close(file_ptr);
    exit(-2);
  }

  /****************************************************************************/
  /* Write out Definitions of Column Headings.                                */
  /****************************************************************************/

  printf("Program Definitions and Units -\n");
  printf("    BX = X Component of the AC Magnetic Field [nT]\n");
  printf("    BY = Y Component of the AC Magnetic Field [nT]\n");
  printf("    BZ = Z Component of the AC Magnetic Field [nT]\n");
  printf(" (RAW) = Raw Compressed Count Rate ");
  printf("in Telemetry (index for error) [unitless]\n");
  printf("\n");
  printf("Notes -\n");
  printf("     1)  -1.0e+31 is used to indicate filled value\n");
  printf("     2)   1.0e+31 is used to indicate excluded value\n");
  printf("     3)  values less than -1.0e+31, between -1.0e-31 and +1.0e-31, ");
  printf("and greater than +1.0e+31\n");
  printf("         are used as fill values at different times during the ");
  printf("processing of telemetry\n");
  printf("     4)  non-zero quality values are for diagnostic purposes only ");
  printf("and indicate processing failure point\n");
  printf("\n");

  /****************************************************************************/
  /* Loop through reading the data records and read them.  terminate the      */
  /* program when the all have been read.                                     */
  /****************************************************************************/

  while ((empty=read(file_ptr,(char*)&data_rec,sizeof(data_rec))) > 0)
  {

    /**************************************************************************/
    /* For the Sample time, convert the millisec per day into hours, minutes  */
    /* and seconds.  Print all starting sweep time values.                    */
    /**************************************************************************/

    spd = data_rec.smsec / 1000;
    hr = spd / 3600;
    mn = (spd - hr * 3600) / 60;
    sc = (spd - hr * 3600 - mn * 60);
    fc = data_rec.smsec - (hr * 3600 + mn * 60 + sc) * 1000;
    printf("Sample Time - %04ld%03ld/%02d:%02d:%02d.%03d (%ld msec)\n",
           data_rec.syear,data_rec.sday,hr,mn,sc,fc,data_rec.smsec);
    printf ("\n");

    /**************************************************************************/
    /* Some O/A parameters are given at the center time of the sweep.  Print  */
    /* these values.                                                          */
    /**************************************************************************/

    printf("Spacecraft Position ->\n");
    if (data_rec.cgalt < 0.0 || data_rec.cgalt > 1000.0)
                              printf("     Spacecraft Altitude = BAD,        ");
    else printf("     Spacecraft Altitude = %6.2f km,  ",data_rec.cgalt);
    if (data_rec.csza < 0.0 || data_rec.csza > 360.0)
                                           printf("Solar Zenith Angle = BAD\n");
    else printf("Solar Zenith Angle = %7.2f deg\n",data_rec.csza);
    if (data_rec.cglat < -90.0 || data_rec.cglat > 90.0)
                               printf("     Geodetic Latitude = BAD,         ");
    else printf("     Geodetic Latitude = %6.2f deg,  ",data_rec.cglat);
    if (data_rec.cglon < 0.0 || data_rec.cglon > 360.0)
                                               printf("East Longitude = BAD\n");
    else printf("East Longitude = %7.2f deg\n",data_rec.cglon);
    if (data_rec.cil600 < -90.0 || data_rec.cil600 > 90.0)
                   printf("     At 600 km: Invariant Latitude = BAD,         ");
    else printf("     At 600 km: Invariant Latitude = %6.2f deg,  ",
                                                               data_rec.cil600);
    if (data_rec.cmst600 < 0.0 || data_rec.cmst600 > 24.0)
                                          printf("Magnetic Solar Time = BAD\n");
    else printf("Magnetic Solar Time = %6.2f hr\n",data_rec.cmst600);
    printf ("\n");

    /**************************************************************************/
    /* Print out the temperatures for this VMAG unit.                         */
    /**************************************************************************/

    printf("Instrument Temperatures ->\n");
    printf("     VMAG Sensor = ");
    if (data_rec.pem_vmag_temp > -1000.0 && data_rec.pem_vmag_temp < 1000.0) 
              printf("%6.2f degC\n", data_rec.pem_vmag_temp);
    else printf("BAD\n");
    printf ("\n");

    /**************************************************************************/
    /* Print out the values of the rotation matrix needed to rotate the VMAG  */
    /* magnetic field vector to the zenith boom.  This rotation needs to      */
    /* occur in order to determine the magnetic field in the frame of the PEM */
    /* zenith boom particle instruments.                                      */
    /**************************************************************************/

    printf("Rotation Matrix From VMAG to ZBOOM ->\n");
    printf("     (1,1) = %9.6f", header_rec.vmag_zboom_11);
    printf("     (1,2) = %9.6f", header_rec.vmag_zboom_12);
    printf("     (1,3) = %9.6f\n", header_rec.vmag_zboom_13);
    printf("     (2,1) = %9.6f", header_rec.vmag_zboom_21);
    printf("     (2,2) = %9.6f", header_rec.vmag_zboom_22);
    printf("     (2,3) = %9.6f\n", header_rec.vmag_zboom_23);
    printf("     (3,1) = %9.6f", header_rec.vmag_zboom_31);
    printf("     (3,2) = %9.6f", header_rec.vmag_zboom_32);
    printf("     (3,3) = %9.6f\n", header_rec.vmag_zboom_33);
    printf ("\n");

    /**************************************************************************/
    /* Print out the values of the rotation matrix needed to rotate the VMAG  */
    /* magnetic field vector from the zenith boom to the UARS spacecraft.     */
    /* This rotation needs to occur in order to determine the magnetic field  */
    /* in the frame of the UARS spacecraft (the same frame as the UARS aspect */
    /* magnetometers and torquer bars).                                       */
    /**************************************************************************/

    printf("Rotation Matrix From ZBOOM to UARS ->\n");
    printf("     (1,1) = %9.6f", header_rec.zboom_uars_11);
    printf("     (1,2) = %9.6f", header_rec.zboom_uars_12);
    printf("     (1,3) = %9.6f\n", header_rec.zboom_uars_13);
    printf("     (2,1) = %9.6f", header_rec.zboom_uars_21);
    printf("     (2,2) = %9.6f", header_rec.zboom_uars_22);
    printf("     (2,3) = %9.6f\n", header_rec.zboom_uars_23);
    printf("     (3,1) = %9.6f", header_rec.zboom_uars_31);
    printf("     (3,2) = %9.6f", header_rec.zboom_uars_32);
    printf("     (3,3) = %9.6f\n", header_rec.zboom_uars_33);
    printf ("\n");

    /**************************************************************************/
    /* Print out the values of the rotation matrix needed to rotate the VMAG  */
    /* magnetic field vector from the UARS spacecraft to Earth Centered       */
    /* Inertial (ECI) coordinates.  This rotation needs to occur in order to  */
    /* determine the magnetic field in the frame of the Earth.                */
    /**************************************************************************/

    printf("Rotation Matrix From UARS to ECI ->\n");
    printf("     (1,1) = %9.6f", data_rec.uars_eci_11);
    printf("     (1,2) = %9.6f", data_rec.uars_eci_12);
    printf("     (1,3) = %9.6f\n", data_rec.uars_eci_13);
    printf("     (2,1) = %9.6f", data_rec.uars_eci_21);
    printf("     (2,2) = %9.6f", data_rec.uars_eci_22);
    printf("     (2,3) = %9.6f\n", data_rec.uars_eci_23);
    printf("     (3,1) = %9.6f", data_rec.uars_eci_31);
    printf("     (3,2) = %9.6f", data_rec.uars_eci_32);
    printf("     (3,3) = %9.6f\n", data_rec.uars_eci_33);
    printf ("\n");

    /**************************************************************************/
    /* Print out each AC component of the PEM VMAG Science Magnetometer.      */
    /**************************************************************************/

    printf("PEM VMAG Science Magnetometer AC ->");
    printf("  QUALITY  X: ");
    if (((int) data_rec.pem_vmag_ac_q[0]) != 0) printf("BAD  ");
    else printf("%03d  ",(int) data_rec.pem_vmag_ac_q[0]);
    printf("  Y: ");
    if (((int) data_rec.pem_vmag_ac_q[1]) != 0) printf("BAD  ");
    else printf("%03d  ",(int) data_rec.pem_vmag_ac_q[1]);
    printf("  Z: ");
    if (((int) data_rec.pem_vmag_ac_q[2]) != 0) printf("BAD  ");
    else printf("%03d  ",(int) data_rec.pem_vmag_ac_q[2]);
    printf ("\n");
    printf("  BX: ");
    if (((int)data_rec.pem_vmag_ac_q[0]) != 0) printf("                      ");
    else printf("%9.2f nT (%06d)  ", data_rec.pem_vmag_ac_nT[0],
                     data_rec.pem_vmag_ac_raw[0]);
    printf("  BY: ");
    if (((int)data_rec.pem_vmag_ac_q[1]) != 0) printf("                      ");
    else printf("%9.2f nT (%06d)  ", data_rec.pem_vmag_ac_nT[1],
                     data_rec.pem_vmag_ac_raw[1]);
    printf("  BZ: ");
    if (((int)data_rec.pem_vmag_ac_q[2]) != 0) printf("\n");
    else printf("%9.2f nT (%06d)\n", data_rec.pem_vmag_ac_nT[2],
                     data_rec.pem_vmag_ac_raw[2]);
    printf ("\n");

    /**************************************************************************/
    /* Looked at the data from all of the VMAG data stored in this record;    */
    /* need another data record.                                              */
    /**************************************************************************/

    printf ("\n");
  }

  /****************************************************************************/
  /* End to data in that file.                                                */
  /****************************************************************************/

  close(file_ptr);
  printf("\nEOF\n");
}
